@@ -7,15 +7,14 @@ import (
 	"errors"
 	"fmt"
 	"os"
-	"runtime"
 
 	"github.com/sirupsen/logrus"
 	"github.com/spf13/cobra"
 
-	"github.com/lima-vm/lima/pkg/autostart"
-	"github.com/lima-vm/lima/pkg/instance"
-	networks "github.com/lima-vm/lima/pkg/networks/reconcile"
-	"github.com/lima-vm/lima/pkg/store"
+	"github.com/lima-vm/lima/v2/pkg/autostart"
+	"github.com/lima-vm/lima/v2/pkg/instance"
+	"github.com/lima-vm/lima/v2/pkg/networks/reconcile"
+	"github.com/lima-vm/lima/v2/pkg/store"
 )
 
 func newDeleteCommand() *cobra.Command {
@@ -33,12 +32,13 @@ func newDeleteCommand() *cobra.Command {
 }
 
 func deleteAction(cmd *cobra.Command, args []string) error {
+	ctx := cmd.Context()
 	force, err := cmd.Flags().GetBool("force")
 	if err != nil {
 		return err
 	}
 	for _, instName := range args {
-		inst, err := store.Inspect(instName)
+		inst, err := store.Inspect(ctx, instName)
 		if err != nil {
 			if errors.Is(err, os.ErrNotExist) {
 				logrus.Warnf("Ignoring non-existent instance %q", instName)
@@ -49,17 +49,18 @@ func deleteAction(cmd *cobra.Command, args []string) error {
 		if err := instance.Delete(cmd.Context(), inst, force); err != nil {
 			return fmt.Errorf("failed to delete instance %q: %w", instName, err)
 		}
-		if runtime.GOOS == "darwin" || runtime.GOOS == "linux" {
-			deleted, err := autostart.DeleteStartAtLoginEntry(runtime.GOOS, instName)
-			if err != nil && !errors.Is(err, os.ErrNotExist) {
-				logrus.WithError(err).Warnf("The autostart file for instance %q does not exist", instName)
-			} else if deleted {
-				logrus.Infof("The autostart file %q has been deleted", autostart.GetFilePath(runtime.GOOS, instName))
+		if registered, err := autostart.IsRegistered(ctx, inst); err != nil && !errors.Is(err, autostart.ErrNotSupported) {
+			logrus.WithError(err).Warnf("Failed to check if the autostart entry for instance %q is registered", instName)
+		} else if registered {
+			if err := autostart.UnregisterFromStartAtLogin(ctx, inst); err != nil {
+				logrus.WithError(err).Warnf("Failed to unregister the autostart entry for instance %q", instName)
+			} else {
+				logrus.Infof("The autostart entry for instance %q has been unregistered", instName)
 			}
 		}
 		logrus.Infof("Deleted %q (%q)", instName, inst.Dir)
 	}
-	return networks.Reconcile(cmd.Context(), "")
+	return reconcile.Reconcile(cmd.Context(), "")
 }
 
 func deleteBashComplete(cmd *cobra.Command, _ []string, _ string) ([]string, cobra.ShellCompDirective) {
