@@ -8,6 +8,7 @@ import (
 	"fmt"
 	"math/bits"
 	"runtime"
+	"slices"
 	"strconv"
 	"strings"
 
@@ -15,6 +16,9 @@ import (
 	"github.com/sirupsen/logrus"
 	"github.com/spf13/cobra"
 	flag "github.com/spf13/pflag"
+
+	"github.com/lima-vm/lima/v2/pkg/localpathutil"
+	"github.com/lima-vm/lima/v2/pkg/registry"
 )
 
 // RegisterEdit registers flags related to in-place YAML modification, for `limactl edit`.
@@ -42,6 +46,8 @@ func RegisterEdit(cmd *cobra.Command, commentPrefix string) {
 	})
 
 	flags.StringSlice("mount", nil, commentPrefix+"Directories to mount, suffix ':w' for writable (Do not specify directories that overlap with the existing mounts)") // colima-compatible
+	flags.StringSlice("mount-only", nil, commentPrefix+"Similar to --mount, but overrides the existing mounts")
+
 	flags.Bool("mount-none", false, commentPrefix+"Remove all mounts")
 
 	flags.String("mount-type", "", commentPrefix+"Mount type (reverse-sshfs, 9p, virtiofs)") // Similar to colima's --mount-type=(sshfs|9p|virtiofs), but "reverse-sshfs" is Lima is called "sshfs" in colima
@@ -60,7 +66,13 @@ func RegisterEdit(cmd *cobra.Command, commentPrefix string) {
 
 	flags.Bool("rosetta", false, commentPrefix+"Enable Rosetta (for vz instances)")
 
-	flags.String("set", "", commentPrefix+"Modify the template inplace, using yq syntax")
+	flags.StringArray("set", []string{}, commentPrefix+"Modify the template inplace, using yq syntax. Can be passed multiple times.")
+
+	flags.Uint16("ssh-port", 0, commentPrefix+"SSH port (0 for random)") // colima-compatible
+	_ = cmd.RegisterFlagCompletionFunc("ssh-port", func(*cobra.Command, []string, string) ([]string, cobra.ShellCompDirective) {
+		// Until Lima v2.0, 60022 was the default SSH port for the instance named "default".
+		return []string{"60022"}, cobra.ShellCompDirectiveNoFileComp
+	})
 
 	// negative performance impact: https://gitlab.com/qemu-project/qemu/-/issues/334
 	flags.Bool("video", false, commentPrefix+"Enable video output (has negative performance impact for QEMU)")
@@ -69,6 +81,15 @@ func RegisterEdit(cmd *cobra.Command, commentPrefix string) {
 	_ = cmd.RegisterFlagCompletionFunc("disk", func(*cobra.Command, []string, string) ([]string, cobra.ShellCompDirective) {
 		return []string{"10", "30", "50", "100", "200"}, cobra.ShellCompDirectiveNoFileComp
 	})
+
+	flags.String("vm-type", "", commentPrefix+"Virtual machine type")
+	_ = cmd.RegisterFlagCompletionFunc("vm-type", func(*cobra.Command, []string, string) ([]string, cobra.ShellCompDirective) {
+		var drivers []string
+		for k := range registry.List() {
+			drivers = append(drivers, k)
+		}
+		return drivers, cobra.ShellCompDirectiveNoFileComp
+	})
 }
 
 // RegisterCreate registers flags related to in-place YAML modification, for `limactl create`.
@@ -86,25 +107,93 @@ func RegisterCreate(cmd *cobra.Command, commentPrefix string) {
 		return []string{"user", "system", "user+system", "none"}, cobra.ShellCompDirectiveNoFileComp
 	})
 
-	flags.String("vm-type", "", commentPrefix+"Virtual machine type (qemu, vz)") // colima-compatible
-	_ = cmd.RegisterFlagCompletionFunc("vm-type", func(*cobra.Command, []string, string) ([]string, cobra.ShellCompDirective) {
-		return []string{"qemu", "vz"}, cobra.ShellCompDirectiveNoFileComp
-	})
-
 	flags.Bool("plain", false, commentPrefix+"Plain mode. Disables mounts, port forwarding, containerd, etc.")
+
+	flags.StringArray("port-forward", nil, commentPrefix+"Port forwards (host:guest), e.g., '8080:80' or '9090:9090,static=true' for static port-forwards")
+	_ = cmd.RegisterFlagCompletionFunc("port-forward", func(*cobra.Command, []string, string) ([]string, cobra.ShellCompDirective) {
+		return []string{"8080:80", "3000:3000", "8080:80,static=true"}, cobra.ShellCompDirectiveNoFileComp
+	})
 }
 
-func defaultExprFunc(expr string) func(v *flag.Flag) (string, error) {
-	return func(v *flag.Flag) (string, error) {
-		return fmt.Sprintf(expr, v.Value), nil
+func defaultExprFunc(expr string) func(v *flag.Flag) ([]string, error) {
+	return func(v *flag.Flag) ([]string, error) {
+		return []string{fmt.Sprintf(expr, v.Value)}, nil
 	}
 }
 
+func ParsePortForward(spec string) (hostPort, guestPort string, isStatic bool, err error) {
+	parts := strings.Split(spec, ",")
+	if len(parts) > 2 {
+		return "", "", false, fmt.Errorf("invalid port forward format %q, expected HOST:GUEST or HOST:GUEST,static=true", spec)
+	}
+
+	portParts := strings.Split(strings.TrimSpace(parts[0]), ":")
+	if len(portParts) != 2 {
+		return "", "", false, fmt.Errorf("invalid port forward format %q, expected HOST:GUEST", parts[0])
+	}
+
+	hostPort = strings.TrimSpace(portParts[0])
+	guestPort = strings.TrimSpace(portParts[1])
+
+	if len(parts) == 2 {
+		staticPart := strings.TrimSpace(parts[1])
+		if strings.HasPrefix(staticPart, "static=") {
+			staticValue := strings.TrimPrefix(staticPart, "static=")
+			isStatic, err = strconv.ParseBool(staticValue)
+			if err != nil {
+				return "", "", false, fmt.Errorf("invalid value for static parameter: %q", staticValue)
+			}
+		} else {
+			return "", "", false, fmt.Errorf("invalid parameter %q, expected 'static=' followed by a boolean value", staticPart)
+		}
+	}
+
+	return hostPort, guestPort, isStatic, nil
+}
+
+func BuildPortForwardExpression(portForwards []string) (string, error) {
+	if len(portForwards) == 0 {
+		return "", nil
+	}
+
+	expr := `.portForwards += [`
+	for i, spec := range portForwards {
+		hostPort, guestPort, isStatic, err := ParsePortForward(spec)
+		if err != nil {
+			return "", err
+		}
+		expr += fmt.Sprintf(`{"guestPort": %q, "hostPort": %q, "static": %v}`, guestPort, hostPort, isStatic)
+		if i < len(portForwards)-1 {
+			expr += ","
+		}
+	}
+	expr += `]`
+	return expr, nil
+}
+
+func buildMountListExpression(ss []string) (string, error) {
+	expr := `[`
+	for i, s := range ss {
+		writable := strings.HasSuffix(s, ":w")
+		loc := strings.TrimSuffix(s, ":w")
+		loc, err := localpathutil.Expand(loc)
+		if err != nil {
+			return "", err
+		}
+		expr += fmt.Sprintf(`{"location": %q, "mountPoint": %q, "writable": %v}`, loc, loc, writable)
+		if i < len(ss)-1 {
+			expr += ","
+		}
+	}
+	expr += `]`
+	return expr, nil
+}
+
 // YQExpressions returns YQ expressions.
 func YQExpressions(flags *flag.FlagSet, newInstance bool) ([]string, error) {
 	type def struct {
 		flagName                 string
-		exprFunc                 func(*flag.Flag) (string, error)
+		exprFunc                 func(*flag.Flag) ([]string, error)
 		onlyValidForNewInstances bool
 		experimental             bool
 	}
@@ -113,10 +202,10 @@ func YQExpressions(flags *flag.FlagSet, newInstance bool) ([]string, error) {
 		{"cpus", d(".cpus = %s"), false, false},
 		{
 			"dns",
-			func(_ *flag.Flag) (string, error) {
+			func(_ *flag.Flag) ([]string, error) {
 				ipSlice, err := flags.GetIPSlice("dns")
 				if err != nil {
-					return "", err
+					return nil, err
 				}
 				expr := `.dns += [`
 				for i, ip := range ipSlice {
@@ -127,7 +216,7 @@ func YQExpressions(flags *flag.FlagSet, newInstance bool) ([]string, error) {
 				}
 				expr += `] | .dns |= unique | .hostResolver.enabled=false`
 				logrus.Warnf("Disabling HostResolver, as custom DNS addresses (%v) are specified", ipSlice)
-				return expr, nil
+				return []string{expr}, nil
 			},
 			false,
 			false,
@@ -135,50 +224,75 @@ func YQExpressions(flags *flag.FlagSet, newInstance bool) ([]string, error) {
 		{"memory", d(".memory = \"%sGiB\""), false, false},
 		{
 			"mount",
-			func(_ *flag.Flag) (string, error) {
+			func(_ *flag.Flag) ([]string, error) {
 				ss, err := flags.GetStringSlice("mount")
+				slices.Reverse(ss)
 				if err != nil {
-					return "", err
+					return nil, err
 				}
-				expr := `.mounts += [`
-				for i, s := range ss {
-					writable := strings.HasSuffix(s, ":w")
-					loc := strings.TrimSuffix(s, ":w")
-					expr += fmt.Sprintf(`{"location": %q, "writable": %v}`, loc, writable)
-					if i < len(ss)-1 {
-						expr += ","
-					}
+				mountListExpr, err := buildMountListExpression(ss)
+				if err != nil {
+					return nil, err
 				}
-				expr += `] | .mounts |= unique_by(.location)`
-				return expr, nil
+				// mount options take precedence over template settings
+				expr := fmt.Sprintf(".mounts = %s + .mounts", mountListExpr)
+				mountOnly, err := flags.GetStringSlice("mount-only")
+				if err != nil {
+					return nil, err
+				}
+				if len(mountOnly) > 0 {
+					return nil, errors.New("flag `--mount` conflicts with `--mount-only`")
+				}
+				return []string{expr}, nil
+			},
+			false,
+			false,
+		},
+		{
+			"mount-only",
+			func(_ *flag.Flag) ([]string, error) {
+				ss, err := flags.GetStringSlice("mount-only")
+				if err != nil {
+					return nil, err
+				}
+				mountListExpr, err := buildMountListExpression(ss)
+				if err != nil {
+					return nil, err
+				}
+				expr := `.mounts = ` + mountListExpr
+				return []string{expr}, nil
 			},
 			false,
 			false,
 		},
 		{
 			"mount-none",
-			func(_ *flag.Flag) (string, error) {
-				ss, err := flags.GetStringSlice("mount")
-				if err != nil {
-					return "", err
+			func(_ *flag.Flag) ([]string, error) {
+				incompatibleFlagNames := []string{"mount", "mount-only"}
+				for _, name := range incompatibleFlagNames {
+					ss, err := flags.GetStringSlice(name)
+					if err != nil {
+						return nil, err
+					}
+					if len(ss) > 0 {
+						return nil, errors.New("flag `--mount-none` conflicts with `" + name + "`")
+					}
 				}
-				if len(ss) > 0 {
-					return "", errors.New("flag `--mount-none` conflicts with `--mount`")
-				}
-				return ".mounts = null", nil
+				return []string{".mounts = null"}, nil
 			},
 			false,
 			false,
 		},
 		{"mount-type", d(".mountType = %q"), false, false},
+		{"vm-type", d(".vmType = %q"), false, false},
 		{"mount-inotify", d(".mountInotify = %s"), false, true},
 		{"mount-writable", d(".mounts[].writable = %s"), false, false},
 		{
 			"network",
-			func(_ *flag.Flag) (string, error) {
+			func(_ *flag.Flag) ([]string, error) {
 				ss, err := flags.GetStringSlice("network")
 				if err != nil {
-					return "", err
+					return nil, err
 				}
 				expr := `.networks += [`
 				for i, s := range ss {
@@ -190,73 +304,92 @@ func YQExpressions(flags *flag.FlagSet, newInstance bool) ([]string, error) {
 						network := strings.TrimPrefix(s, "lima:")
 						expr += fmt.Sprintf(`{"lima": %q}`, network)
 					default:
-						return "", fmt.Errorf("network name must be \"vzNAT\" or \"lima:*\", got %q", s)
+						return nil, fmt.Errorf("network name must be \"vzNAT\" or \"lima:*\", got %q", s)
 					}
 					if i < len(ss)-1 {
 						expr += ","
 					}
 				}
 				expr += `] | .networks |= unique_by(.lima)`
-				return expr, nil
+				return []string{expr}, nil
 			},
 			false,
 			false,
 		},
+
 		{
 			"rosetta",
-			func(_ *flag.Flag) (string, error) {
+			func(_ *flag.Flag) ([]string, error) {
 				b, err := flags.GetBool("rosetta")
 				if err != nil {
-					return "", err
+					return nil, err
 				}
-				return fmt.Sprintf(".rosetta.enabled = %v | .rosetta.binfmt = %v", b, b), nil
+				return []string{fmt.Sprintf(".vmOpts.vz.rosetta.enabled = %v | .vmOpts.vz.rosetta.binfmt = %v", b, b)}, nil
 			},
 			false,
 			false,
 		},
-		{"set", d("%s"), false, false},
+		{"set", func(v *flag.Flag) ([]string, error) {
+			return v.Value.(flag.SliceValue).GetSlice(), nil
+		}, false, false},
 		{
 			"video",
-			func(_ *flag.Flag) (string, error) {
+			func(_ *flag.Flag) ([]string, error) {
 				b, err := flags.GetBool("video")
 				if err != nil {
-					return "", err
+					return nil, err
 				}
 				if b {
-					return ".video.display = \"default\"", nil
+					return []string{".video.display = \"default\""}, nil
 				}
-				return ".video.display = \"none\"", nil
+				return []string{".video.display = \"none\""}, nil
 			},
 			false,
 			false,
 		},
+		{"ssh-port", d(".ssh.localPort = %s"), false, false},
 		{"arch", d(".arch = %q"), true, false},
 		{
 			"containerd",
-			func(_ *flag.Flag) (string, error) {
+			func(_ *flag.Flag) ([]string, error) {
 				s, err := flags.GetString("containerd")
 				if err != nil {
-					return "", err
+					return nil, err
 				}
 				switch s {
 				case "user":
-					return `.containerd.user = true | .containerd.system = false`, nil
+					return []string{`.containerd.user = true | .containerd.system = false`}, nil
 				case "system":
-					return `.containerd.user = false | .containerd.system = true`, nil
+					return []string{`.containerd.user = false | .containerd.system = true`}, nil
 				case "user+system", "system+user":
-					return `.containerd.user = true | .containerd.system = true`, nil
+					return []string{`.containerd.user = true | .containerd.system = true`}, nil
 				case "none":
-					return `.containerd.user = false | .containerd.system = false`, nil
+					return []string{`.containerd.user = false | .containerd.system = false`}, nil
 				default:
-					return "", fmt.Errorf(`expected one of ["user", "system", "user+system", "none"], got %q`, s)
+					return nil, fmt.Errorf(`expected one of ["user", "system", "user+system", "none"], got %q`, s)
 				}
 			},
 			true,
 			false,
 		},
 		{"disk", d(".disk= \"%sGiB\""), false, false},
-		{"vm-type", d(".vmType = %q"), true, false},
 		{"plain", d(".plain = %s"), true, false},
+		{
+			"port-forward",
+			func(_ *flag.Flag) ([]string, error) {
+				ss, err := flags.GetStringArray("port-forward")
+				if err != nil {
+					return nil, err
+				}
+				value, err := BuildPortForwardExpression(ss)
+				if err != nil {
+					return nil, err
+				}
+				return []string{value}, nil
+			},
+			false,
+			false,
+		},
 	}
 	var exprs []string
 	for _, def := range defs {
@@ -270,11 +403,11 @@ func YQExpressions(flags *flag.FlagSet, newInstance bool) ([]string, error) {
 					def.flagName, def.flagName, v.Value.String())
 				continue
 			}
-			expr, err := def.exprFunc(v)
+			newExprs, err := def.exprFunc(v)
 			if err != nil {
 				return exprs, fmt.Errorf("error while processing flag %q: %w", def.flagName, err)
 			}
-			exprs = append(exprs, expr)
+			exprs = append(exprs, newExprs...)
 		}
 	}
 	return exprs, nil
