@@ -18,8 +18,9 @@ import (
 	"github.com/sirupsen/logrus"
 	"github.com/spf13/cobra"
 
-	"github.com/lima-vm/lima/pkg/hostagent"
-	"github.com/lima-vm/lima/pkg/hostagent/api/server"
+	"github.com/lima-vm/lima/v2/pkg/hostagent"
+	"github.com/lima-vm/lima/v2/pkg/hostagent/api/server"
+	"github.com/lima-vm/lima/v2/pkg/store"
 )
 
 func newHostagentCommand() *cobra.Command {
@@ -35,17 +36,21 @@ func newHostagentCommand() *cobra.Command {
 	hostagentCommand.Flags().Bool("run-gui", false, "Run GUI synchronously within hostagent")
 	hostagentCommand.Flags().String("guestagent", "", "Local file path (not URL) of lima-guestagent.OS-ARCH[.gz]")
 	hostagentCommand.Flags().String("nerdctl-archive", "", "Local file path (not URL) of nerdctl-full-VERSION-GOOS-GOARCH.tar.gz")
+	hostagentCommand.Flags().Bool("progress", false, "Show provision script progress by monitoring cloud-init logs")
 	return hostagentCommand
 }
 
 func hostagentAction(cmd *cobra.Command, args []string) error {
+	ctx := cmd.Context()
 	pidfile, err := cmd.Flags().GetString("pidfile")
 	if err != nil {
 		return err
 	}
 	if pidfile != "" {
-		if _, err := os.Stat(pidfile); !errors.Is(err, os.ErrNotExist) {
-			return fmt.Errorf("pidfile %q already exists", pidfile)
+		if existingPID, err := store.ReadPIDFile(pidfile); existingPID != 0 {
+			return fmt.Errorf("another hostagent may already be running with pid %d (pidfile %q)", existingPID, pidfile)
+		} else if err != nil {
+			return fmt.Errorf("failed to determine if another hostagent is running: %w", err)
 		}
 		if err := os.WriteFile(pidfile, []byte(strconv.Itoa(os.Getpid())+"\n"), 0o644); err != nil {
 			return err
@@ -94,7 +99,14 @@ func hostagentAction(cmd *cobra.Command, args []string) error {
 	if nerdctlArchive != "" {
 		opts = append(opts, hostagent.WithNerdctlArchive(nerdctlArchive))
 	}
-	ha, err := hostagent.New(instName, stdout, signalCh, opts...)
+	showProgress, err := cmd.Flags().GetBool("progress")
+	if err != nil {
+		return err
+	}
+	if showProgress {
+		opts = append(opts, hostagent.WithCloudInitProgress(showProgress))
+	}
+	ha, err := hostagent.New(ctx, instName, stdout, signalCh, opts...)
 	if err != nil {
 		return err
 	}
@@ -109,18 +121,18 @@ func hostagentAction(cmd *cobra.Command, args []string) error {
 	if err != nil {
 		return err
 	}
-	l, err := net.Listen("unix", socket)
+	var lc net.ListenConfig
+	l, err := lc.Listen(ctx, "unix", socket)
 	logrus.Infof("hostagent socket created at %s", socket)
 	if err != nil {
 		return err
 	}
 	go func() {
-		defer os.RemoveAll(socket)
-		defer srv.Close()
 		if serveErr := srv.Serve(l); serveErr != http.ErrServerClosed {
 			logrus.WithError(serveErr).Warn("hostagent API server exited with an error")
 		}
 	}()
+	defer srv.Close()
 	return ha.Run(cmd.Context())
 }
 
