@@ -15,11 +15,14 @@ import (
 	"github.com/sirupsen/logrus"
 	"github.com/spf13/cobra"
 
-	"github.com/lima-vm/lima/pkg/debugutil"
-	"github.com/lima-vm/lima/pkg/fsutil"
-	"github.com/lima-vm/lima/pkg/osutil"
-	"github.com/lima-vm/lima/pkg/store/dirnames"
-	"github.com/lima-vm/lima/pkg/version"
+	"github.com/lima-vm/lima/v2/cmd/yq"
+	"github.com/lima-vm/lima/v2/pkg/debugutil"
+	"github.com/lima-vm/lima/v2/pkg/driver/external/server"
+	"github.com/lima-vm/lima/v2/pkg/fsutil"
+	"github.com/lima-vm/lima/v2/pkg/limatype/dirnames"
+	"github.com/lima-vm/lima/v2/pkg/osutil"
+	"github.com/lima-vm/lima/v2/pkg/plugins"
+	"github.com/lima-vm/lima/v2/pkg/version"
 )
 
 const (
@@ -29,6 +32,11 @@ const (
 )
 
 func main() {
+	if os.Geteuid() == 0 && (len(os.Args) < 2 || os.Args[1] != "generate-doc") {
+		panic("must not run as the root user")
+	}
+
+	yq.MaybeRunYQ()
 	if runtime.GOOS == "windows" {
 		extras, hasExtra := os.LookupEnv("_LIMA_WINDOWS_EXTRA_PATH")
 		if hasExtra && strings.TrimSpace(extras) != "" {
@@ -39,12 +47,49 @@ func main() {
 			}
 		}
 	}
-	if err := newApp().Execute(); err != nil {
-		handleExitCoder(err)
+	err := newApp().Execute()
+	server.StopAllExternalDrivers()
+	osutil.HandleExitError(err)
+	if err != nil {
 		logrus.Fatal(err)
 	}
 }
 
+func processGlobalFlags(rootCmd *cobra.Command) error {
+	// --log-level will override --debug, but will not reset debugutil.Debug
+	if debug, _ := rootCmd.Flags().GetBool("debug"); debug {
+		logrus.SetLevel(logrus.DebugLevel)
+		debugutil.Debug = true
+	}
+
+	l, _ := rootCmd.Flags().GetString("log-level")
+	if l != "" {
+		lvl, err := logrus.ParseLevel(l)
+		if err != nil {
+			return err
+		}
+		logrus.SetLevel(lvl)
+	}
+
+	logFormat, _ := rootCmd.Flags().GetString("log-format")
+	switch logFormat {
+	case "json":
+		formatter := new(logrus.JSONFormatter)
+		logrus.StandardLogger().SetFormatter(formatter)
+	case "text":
+		// logrus use text format by default.
+		if runtime.GOOS == "windows" && isatty.IsCygwinTerminal(os.Stderr.Fd()) {
+			formatter := new(logrus.TextFormatter)
+			// the default setting does not recognize cygwin on windows
+			formatter.ForceColors = true
+			logrus.StandardLogger().SetFormatter(formatter)
+		}
+	default:
+		return fmt.Errorf("unsupported log-format: %q", logFormat)
+	}
+	return nil
+}
+
 func newApp() *cobra.Command {
 	templatesDir := "$PREFIX/share/lima/templates"
 	if exe, err := os.Executable(); err == nil {
@@ -81,36 +126,8 @@ func newApp() *cobra.Command {
 	rootCmd.PersistentFlags().Bool("tty", isatty.IsTerminal(os.Stdout.Fd()), "Enable TUI interactions such as opening an editor. Defaults to true when stdout is a terminal. Set to false for automation.")
 	rootCmd.PersistentFlags().BoolP("yes", "y", false, "Alias of --tty=false")
 	rootCmd.PersistentPreRunE = func(cmd *cobra.Command, _ []string) error {
-		l, _ := cmd.Flags().GetString("log-level")
-		if l != "" {
-			lvl, err := logrus.ParseLevel(l)
-			if err != nil {
-				return err
-			}
-			logrus.SetLevel(lvl)
-		}
-
-		logFormat, _ := cmd.Flags().GetString("log-format")
-		switch logFormat {
-		case "json":
-			formatter := new(logrus.JSONFormatter)
-			logrus.StandardLogger().SetFormatter(formatter)
-		case "text":
-			// logrus use text format by default.
-			if runtime.GOOS == "windows" && isatty.IsCygwinTerminal(os.Stderr.Fd()) {
-				formatter := new(logrus.TextFormatter)
-				// the default setting does not recognize cygwin on windows
-				formatter.ForceColors = true
-				logrus.StandardLogger().SetFormatter(formatter)
-			}
-		default:
-			return fmt.Errorf("unsupported log-format: %q", logFormat)
-		}
-
-		debug, _ := cmd.Flags().GetBool("debug")
-		if debug {
-			logrus.SetLevel(logrus.DebugLevel)
-			debugutil.Debug = true
+		if err := processGlobalFlags(rootCmd); err != nil {
+			return err
 		}
 
 		if osutil.IsBeingRosettaTranslated() && cmd.Parent().Name() != "completion" && cmd.Name() != "generate-doc" && cmd.Name() != "validate" {
@@ -119,21 +136,12 @@ func newApp() *cobra.Command {
 			return errors.New("limactl is running under rosetta, please reinstall lima with native arch")
 		}
 
-		if os.Geteuid() == 0 && cmd.Name() != "generate-doc" {
-			return errors.New("must not run as the root user")
-		}
 		// Make sure either $HOME or $LIMA_HOME is defined, so we don't need
 		// to check for errors later
 		dir, err := dirnames.LimaDir()
 		if err != nil {
 			return err
 		}
-		// Make sure that directory is on a local filesystem, not on NFS
-		// if the directory does not yet exist, check the home directory
-		_, err = os.Stat(dir)
-		if errors.Is(err, os.ErrNotExist) {
-			dir = filepath.Dir(dir)
-		}
 		nfs, err := fsutil.IsNFS(dir)
 		if err != nil {
 			return err
@@ -147,6 +155,11 @@ func newApp() *cobra.Command {
 		}
 
 		if cmd.Flags().Changed("yes") {
+			switch cmd.Name() {
+			case "clone", "edit", "rename":
+				logrus.Warn("--yes flag is deprecated (--tty=false is still supported and works in the same way. Also consider using --start)")
+			}
+
 			// Sets the value of the yesValue flag by using the yes flag.
 			yesValue, _ := cmd.Flags().GetBool("yes")
 			if yesValue {
@@ -161,6 +174,8 @@ func newApp() *cobra.Command {
 	}
 	rootCmd.AddGroup(&cobra.Group{ID: "basic", Title: "Basic Commands:"})
 	rootCmd.AddGroup(&cobra.Group{ID: "advanced", Title: "Advanced Commands:"})
+	rootCmd.AddGroup(&cobra.Group{ID: "plugin", Title: "Available Plugins (Experimental):"})
+
 	rootCmd.AddCommand(
 		newCreateCommand(),
 		newStartCommand(),
@@ -172,6 +187,7 @@ func newApp() *cobra.Command {
 		newValidateCommand(),
 		newPruneCommand(),
 		newHostagentCommand(),
+		newGuestInstallCommand(),
 		newInfoCommand(),
 		newShowSSHCommand(),
 		newDebugCommand(),
@@ -191,24 +207,64 @@ func newApp() *cobra.Command {
 		newStartAtLoginCommand(),
 		newNetworkCommand(),
 		newCloneCommand(),
+		newRenameCommand(),
 	)
+	addPluginCommands(rootCmd)
 
 	return rootCmd
 }
 
-type ExitCoder interface {
-	error
-	ExitCode() int
-}
+func addPluginCommands(rootCmd *cobra.Command) {
+	// The global options are only processed when rootCmd.Execute() is called.
+	// Let's take a sneak peek here to help debug the plugin discovery code.
+	if len(os.Args) > 1 && os.Args[1] == "--debug" {
+		logrus.SetLevel(logrus.DebugLevel)
+	}
 
-func handleExitCoder(err error) {
-	if err == nil {
+	allPlugins, err := plugins.Discover()
+	if err != nil {
+		logrus.Warnf("Failed to discover plugins: %v", err)
 		return
 	}
 
-	if exitErr, ok := err.(ExitCoder); ok {
-		os.Exit(exitErr.ExitCode()) //nolint:revive // it's intentional to call os.Exit in this function
-		return
+	for _, plugin := range allPlugins {
+		pluginCmd := &cobra.Command{
+			Use:                plugin.Name,
+			Short:              plugin.Description,
+			GroupID:            "plugin",
+			DisableFlagParsing: true,
+			SilenceErrors:      true,
+			SilenceUsage:       true,
+			PreRunE: func(*cobra.Command, []string) error {
+				for i, arg := range os.Args {
+					if arg == plugin.Name {
+						// parse global options but ignore plugin options
+						err := rootCmd.ParseFlags(os.Args[1:i])
+						if err == nil {
+							err = processGlobalFlags(rootCmd)
+						}
+						return err
+					}
+				}
+				// unreachable
+				return nil
+			},
+			Run: func(cmd *cobra.Command, _ []string) {
+				for i, arg := range os.Args {
+					if arg == plugin.Name {
+						// ignore global options
+						plugin.Run(cmd.Context(), os.Args[i+1:])
+						// plugin.Run() never returns because it calls os.Exit()
+					}
+				}
+				// unreachable
+			},
+		}
+		// Don't show the url scheme helper in the help output.
+		if strings.HasPrefix(plugin.Name, "url-") {
+			pluginCmd.Hidden = true
+		}
+		rootCmd.AddCommand(pluginCmd)
 	}
 }
 
