@@ -14,24 +14,31 @@ import (
 
 	"github.com/sirupsen/logrus"
 	"github.com/spf13/cobra"
+	"github.com/spf13/pflag"
 
-	"github.com/lima-vm/lima/cmd/limactl/editflags"
-	"github.com/lima-vm/lima/pkg/editutil"
-	"github.com/lima-vm/lima/pkg/instance"
-	"github.com/lima-vm/lima/pkg/limatmpl"
-	"github.com/lima-vm/lima/pkg/limayaml"
-	networks "github.com/lima-vm/lima/pkg/networks/reconcile"
-	"github.com/lima-vm/lima/pkg/store"
-	"github.com/lima-vm/lima/pkg/store/filenames"
-	"github.com/lima-vm/lima/pkg/templatestore"
-	"github.com/lima-vm/lima/pkg/uiutil"
-	"github.com/lima-vm/lima/pkg/yqutil"
+	"github.com/lima-vm/lima/v2/cmd/limactl/editflags"
+	"github.com/lima-vm/lima/v2/pkg/autostart"
+	"github.com/lima-vm/lima/v2/pkg/driverutil"
+	"github.com/lima-vm/lima/v2/pkg/editutil"
+	"github.com/lima-vm/lima/v2/pkg/instance"
+	"github.com/lima-vm/lima/v2/pkg/limatmpl"
+	"github.com/lima-vm/lima/v2/pkg/limatype"
+	"github.com/lima-vm/lima/v2/pkg/limatype/dirnames"
+	"github.com/lima-vm/lima/v2/pkg/limatype/filenames"
+	"github.com/lima-vm/lima/v2/pkg/limayaml"
+	"github.com/lima-vm/lima/v2/pkg/networks/reconcile"
+	"github.com/lima-vm/lima/v2/pkg/registry"
+	"github.com/lima-vm/lima/v2/pkg/store"
+	"github.com/lima-vm/lima/v2/pkg/templatestore"
+	"github.com/lima-vm/lima/v2/pkg/uiutil"
+	"github.com/lima-vm/lima/v2/pkg/yqutil"
 )
 
 func registerCreateFlags(cmd *cobra.Command, commentPrefix string) {
 	flags := cmd.Flags()
 	flags.String("name", "", commentPrefix+"Override the instance name")
 	flags.Bool("list-templates", false, commentPrefix+"List available templates and exit")
+	flags.Bool("list-drivers", false, commentPrefix+"List available drivers and exit")
 	editflags.RegisterCreate(cmd, commentPrefix)
 }
 
@@ -39,29 +46,29 @@ func newCreateCommand() *cobra.Command {
 	createCommand := &cobra.Command{
 		Use: "create FILE.yaml|URL",
 		Example: `
-To create an instance "default" from the default Ubuntu template:
-$ limactl create
+  To create an instance "default" from the default Ubuntu template:
+  $ limactl create
 
-To create an instance "default" from a template "docker":
-$ limactl create --name=default template://docker
+  To create an instance "default" from a template "docker":
+  $ limactl create --name=default template:docker
 
-To create an instance "default" with modified parameters:
-$ limactl create --cpus=2 --memory=2
+  To create an instance "default" with modified parameters:
+  $ limactl create --cpus=2 --memory=2
 
-To create an instance "default" with yq expressions:
-$ limactl create --set='.cpus = 2 | .memory = "2GiB"'
+  To create an instance "default" with yq expressions:
+  $ limactl create --set='.cpus = 2 | .memory = "2GiB"'
 
-To see the template list:
-$ limactl create --list-templates
+  To see the template list:
+  $ limactl create --list-templates
 
-To create an instance "default" from a local file:
-$ limactl create --name=default /usr/local/share/lima/templates/fedora.yaml
+  To create an instance "default" from a local file:
+  $ limactl create --name=default /usr/local/share/lima/templates/fedora.yaml
 
-To create an instance "default" from a remote URL (use carefully, with a trustable source):
-$ limactl create --name=default https://raw.githubusercontent.com/lima-vm/lima/master/templates/alpine.yaml
+  To create an instance "default" from a remote URL (use carefully, with a trustable source):
+  $ limactl create --name=default https://raw.githubusercontent.com/lima-vm/lima/master/templates/alpine.yaml
 
-To create an instance "local" from a template passed to stdin (--name parameter is required):
-$ cat template.yaml | limactl create --name=local -
+  To create an instance "local" from a template passed to stdin (--name parameter is required):
+  $ cat template.yaml | limactl create --name=local -
 `,
 		Short:             "Create an instance of Lima",
 		Args:              WrapArgsError(cobra.MaximumNArgs(1)),
@@ -77,14 +84,11 @@ func newStartCommand() *cobra.Command {
 	startCommand := &cobra.Command{
 		Use: "start NAME|FILE.yaml|URL",
 		Example: `
-To create an instance "default" (if not created yet) from the default Ubuntu template, and start it:
-$ limactl start
+  To create an instance "default" (if not created yet) from the default Ubuntu template, and start it:
+  $ limactl start
 
-To create an instance "default" from a template "docker", and start it:
-$ limactl start --name=default template://docker
-
-'limactl start' also accepts the 'limactl create' flags such as '--set'.
-See the examples in 'limactl create --help'.
+  To create an instance "default" from a template "docker", and start it:
+  $ limactl start --name=default template:docker
 `,
 		Short:             "Start an instance of Lima",
 		Args:              WrapArgsError(cobra.MaximumNArgs(1)),
@@ -97,10 +101,114 @@ See the examples in 'limactl create --help'.
 		startCommand.Flags().Bool("foreground", false, "Run the hostagent in the foreground")
 	}
 	startCommand.Flags().Duration("timeout", instance.DefaultWatchHostAgentEventsTimeout, "Duration to wait for the instance to be running before timing out")
+	startCommand.Flags().Bool("progress", false, "Show provision script progress by tailing cloud-init logs")
+	startCommand.SetHelpFunc(func(cmd *cobra.Command, _ []string) {
+		printCommandSummary(cmd)
+
+		allFlags, createFlags := collectFlags(cmd)
+		printFlags(allFlags, createFlags)
+
+		printGlobalFlags(cmd)
+	})
+
 	return startCommand
 }
 
-func loadOrCreateInstance(cmd *cobra.Command, args []string, createOnly bool) (*store.Instance, error) {
+func printCommandSummary(cmd *cobra.Command) {
+	fmt.Fprintf(cmd.OutOrStdout(), "%s\n\n", cmd.Short)
+	fmt.Fprintf(cmd.OutOrStdout(), "Usage:\n  %s\n\n", cmd.UseLine())
+
+	if cmd.Example != "" {
+		fmt.Fprintf(cmd.OutOrStdout(), "Examples:\n%s\n\n", cmd.Example)
+	}
+}
+
+func getFlagType(flag *pflag.Flag) string {
+	switch flag.Value.Type() {
+	case "bool":
+		return ""
+	case "string":
+		return "string"
+	case "int":
+		return "int"
+	case "duration":
+		return "duration"
+	case "stringSlice", "stringArray":
+		return "strings"
+	case "ipSlice":
+		return "ipSlice"
+	case "uint16":
+		return "uint16"
+	case "float32":
+		return "float32"
+	default:
+		return flag.Value.Type()
+	}
+}
+
+func formatFlag(flag *pflag.Flag) (flagName, shorthand string) {
+	flagName = "--" + flag.Name
+
+	if flag.Shorthand != "" {
+		shorthand = "-" + flag.Shorthand
+	}
+
+	flagType := getFlagType(flag)
+	if flagType != "" {
+		flagName += " " + flagType
+	}
+
+	return flagName, shorthand
+}
+
+func collectFlags(cmd *cobra.Command) (allFlags, createFlags []string) {
+	cmd.LocalFlags().VisitAll(func(flag *pflag.Flag) {
+		flagName, shorthand := formatFlag(flag)
+		flagUsage := flag.Usage
+
+		var formattedFlag string
+		if shorthand != "" {
+			formattedFlag = fmt.Sprintf("  %s, %s", shorthand, flagName)
+		} else {
+			formattedFlag = fmt.Sprintf("      %s", flagName)
+		}
+
+		if strings.HasPrefix(flagUsage, "[limactl create]") {
+			cleanUsage := strings.TrimPrefix(flagUsage, "[limactl create] ")
+			createFlags = append(createFlags, fmt.Sprintf("%-25s %s", formattedFlag, cleanUsage))
+		} else {
+			allFlags = append(allFlags, fmt.Sprintf("%-25s %s", formattedFlag, flagUsage))
+		}
+	})
+	return allFlags, createFlags
+}
+
+func printFlags(allFlags, createFlags []string) {
+	if len(allFlags) > 0 {
+		fmt.Fprint(os.Stdout, "Flags:\n")
+		for _, flag := range allFlags {
+			fmt.Fprintln(os.Stdout, flag)
+		}
+		fmt.Fprint(os.Stdout, "\n")
+	}
+
+	if len(createFlags) > 0 {
+		fmt.Fprint(os.Stdout, "Flags inherited from `limactl create`:\n")
+		for _, flag := range createFlags {
+			fmt.Fprintln(os.Stdout, flag)
+		}
+		fmt.Fprint(os.Stdout, "\n")
+	}
+}
+
+func printGlobalFlags(cmd *cobra.Command) {
+	if cmd.HasAvailableInheritedFlags() {
+		fmt.Fprintf(cmd.OutOrStdout(), "Global Flags:\n%s", cmd.InheritedFlags().FlagUsages())
+	}
+}
+
+func loadOrCreateInstance(cmd *cobra.Command, args []string, createOnly bool) (*limatype.Instance, error) {
+	ctx := cmd.Context()
 	var arg string // can be empty
 	if len(args) > 0 {
 		arg = args[0]
@@ -119,7 +227,7 @@ func loadOrCreateInstance(cmd *cobra.Command, args []string, createOnly bool) (*
 		return nil, err
 	}
 	if name != "" {
-		err := store.ValidateInstName(name)
+		err := dirnames.ValidateInstName(name)
 		if err != nil {
 			return nil, err
 		}
@@ -127,19 +235,19 @@ func loadOrCreateInstance(cmd *cobra.Command, args []string, createOnly bool) (*
 	if isTemplateURL, templateName := limatmpl.SeemsTemplateURL(arg); isTemplateURL {
 		switch templateName {
 		case "experimental/vz":
-			logrus.Warn("template://experimental/vz was merged into the default template in Lima v1.0. See also <https://lima-vm.io/docs/config/vmtype/>.")
+			logrus.Warn("template:experimental/vz was merged into the default template in Lima v1.0. See also <https://lima-vm.io/docs/config/vmtype/>.")
 		case "experimental/riscv64":
-			logrus.Warn("template://experimental/riscv64 was merged into the default template in Lima v1.0. Use `limactl create --arch=riscv64 template://default` instead.")
+			logrus.Warn("template:experimental/riscv64 was merged into the default template in Lima v1.0. Use `limactl create --arch=riscv64 template:default` instead.")
 		case "experimental/armv7l":
-			logrus.Warn("template://experimental/armv7l was merged into the default template in Lima v1.0. Use `limactl create --arch=armv7l template://default` instead.")
+			logrus.Warn("template:experimental/armv7l was merged into the default template in Lima v1.0. Use `limactl create --arch=armv7l template:default` instead.")
 		case "vmnet":
-			logrus.Warn("template://vmnet was removed in Lima v1.0. Use `limactl create --network=lima:shared template://default` instead. See also <https://lima-vm.io/docs/config/network/>.")
+			logrus.Warn("template:vmnet was removed in Lima v1.0. Use `limactl create --network=lima:shared template:default` instead. See also <https://lima-vm.io/docs/config/network/>.")
 		case "experimental/net-user-v2":
-			logrus.Warn("template://experimental/net-user-v2 was removed in Lima v1.0. Use `limactl create --network=lima:user-v2 template://default` instead. See also <https://lima-vm.io/docs/config/network/>.")
+			logrus.Warn("template:experimental/net-user-v2 was removed in Lima v1.0. Use `limactl create --network=lima:user-v2 template:default` instead. See also <https://lima-vm.io/docs/config/network/>.")
 		case "experimental/9p":
-			logrus.Warn("template://experimental/9p was removed in Lima v1.0. Use `limactl create --vm-type=qemu --mount-type=9p template://default` instead. See also <https://lima-vm.io/docs/config/mount/>.")
+			logrus.Warn("template:experimental/9p was removed in Lima v1.0. Use `limactl create --vm-type=qemu --mount-type=9p template:default` instead. See also <https://lima-vm.io/docs/config/mount/>.")
 		case "experimental/virtiofs-linux":
-			logrus.Warn("template://experimental/virtiofs-linux was removed in Lima v1.0. Use `limactl create --mount-type=virtiofs template://default` instead. See also <https://lima-vm.io/docs/config/mount/>.")
+			logrus.Warn("template:experimental/virtiofs-linux was removed in Lima v1.0. Use `limactl create --mount-type=virtiofs template:default` instead. See also <https://lima-vm.io/docs/config/mount/>.")
 		}
 	}
 	if arg == "-" {
@@ -154,7 +262,7 @@ func loadOrCreateInstance(cmd *cobra.Command, args []string, createOnly bool) (*
 		tty = false
 	}
 	var tmpl *limatmpl.Template
-	if err := store.ValidateInstName(arg); arg == "" || err == nil {
+	if err := dirnames.ValidateInstName(arg); arg == "" || err == nil {
 		tmpl = &limatmpl.Template{Name: name}
 		if arg == "" {
 			if name == "" {
@@ -168,7 +276,7 @@ func loadOrCreateInstance(cmd *cobra.Command, args []string, createOnly bool) (*
 			tmpl.Name = arg
 		}
 		// store.Inspect() will validate the template name (in case it has been set to arg)
-		inst, err := store.Inspect(tmpl.Name)
+		inst, err := store.Inspect(ctx, tmpl.Name)
 		if err == nil {
 			if createOnly {
 				return nil, fmt.Errorf("instance %q already exists", tmpl.Name)
@@ -180,7 +288,7 @@ func loadOrCreateInstance(cmd *cobra.Command, args []string, createOnly bool) (*
 			}
 			if len(yqExprs) > 0 {
 				yq := yqutil.Join(yqExprs)
-				inst, err = applyYQExpressionToExistingInstance(inst, yq)
+				inst, err = applyYQExpressionToExistingInstance(ctx, inst, yq)
 				if err != nil {
 					return nil, fmt.Errorf("failed to apply yq expression %q to instance %q: %w", yq, tmpl.Name, err)
 				}
@@ -191,8 +299,8 @@ func loadOrCreateInstance(cmd *cobra.Command, args []string, createOnly bool) (*
 			return nil, err
 		}
 		if arg != "" && arg != DefaultInstanceName {
-			logrus.Infof("Creating an instance %q from template://default (Not from template://%s)", tmpl.Name, tmpl.Name)
-			logrus.Warnf("This form is deprecated. Use `limactl create --name=%s template://default` instead", tmpl.Name)
+			logrus.Infof("Creating an instance %q from template:default (Not from template:%s)", tmpl.Name, tmpl.Name)
+			logrus.Warnf("This form is deprecated. Use `limactl create --name=%s template:default` instead", tmpl.Name)
 		}
 		// Read the default template for creating a new instance
 		tmpl.Bytes, err = templatestore.Read(templatestore.Default)
@@ -206,10 +314,10 @@ func loadOrCreateInstance(cmd *cobra.Command, args []string, createOnly bool) (*
 		}
 		if createOnly {
 			// store.Inspect() will also validate the instance name
-			if _, err := store.Inspect(tmpl.Name); err == nil {
+			if _, err := store.Inspect(ctx, tmpl.Name); err == nil {
 				return nil, fmt.Errorf("instance %q already exists", tmpl.Name)
 			}
-		} else if err := store.ValidateInstName(tmpl.Name); err != nil {
+		} else if err := dirnames.ValidateInstName(tmpl.Name); err != nil {
 			return nil, err
 		}
 	}
@@ -238,7 +346,7 @@ func loadOrCreateInstance(cmd *cobra.Command, args []string, createOnly bool) (*
 	return instance.Create(cmd.Context(), tmpl.Name, tmpl.Bytes, saveBrokenYAML)
 }
 
-func applyYQExpressionToExistingInstance(inst *store.Instance, yq string) (*store.Instance, error) {
+func applyYQExpressionToExistingInstance(ctx context.Context, inst *limatype.Instance, yq string) (*limatype.Instance, error) {
 	if strings.TrimSpace(yq) == "" {
 		return inst, nil
 	}
@@ -252,10 +360,13 @@ func applyYQExpressionToExistingInstance(inst *store.Instance, yq string) (*stor
 	if err != nil {
 		return nil, err
 	}
-	y, err := limayaml.Load(yBytes, filePath)
+	y, err := limayaml.Load(ctx, yBytes, filePath)
 	if err != nil {
 		return nil, err
 	}
+	if err := driverutil.ResolveVMType(ctx, y, filePath); err != nil {
+		return nil, fmt.Errorf("failed to resolve vm for %q: %w", filePath, err)
+	}
 	if err := limayaml.Validate(y, true); err != nil {
 		rejectedYAML := "lima.REJECTED.yaml"
 		if writeErr := os.WriteFile(rejectedYAML, yBytes, 0o644); writeErr != nil {
@@ -268,7 +379,7 @@ func applyYQExpressionToExistingInstance(inst *store.Instance, yq string) (*stor
 		return nil, err
 	}
 	// Reload
-	return store.Inspect(inst.Name)
+	return store.Inspect(ctx, inst.Name)
 }
 
 func modifyInPlace(st *limatmpl.Template, yq string) error {
@@ -328,7 +439,7 @@ func chooseNextCreatorState(ctx context.Context, tmpl *limatmpl.Template, yq str
 			hdr += "\n"
 			hdr += editutil.GenerateEditorWarningHeader()
 			var err error
-			tmpl.Bytes, err = editutil.OpenEditor(tmpl.Bytes, hdr)
+			tmpl.Bytes, err = editutil.OpenEditor(ctx, tmpl.Bytes, hdr)
 			tmpl.Config = nil
 			if err != nil {
 				return tmpl, err
@@ -398,6 +509,14 @@ func createStartActionCommon(cmd *cobra.Command, _ []string) (exit bool, err err
 			_, _ = fmt.Fprintln(w, f.Name)
 		}
 		return true, nil
+	} else if listDrivers, err := cmd.Flags().GetBool("list-drivers"); err != nil {
+		return true, err
+	} else if listDrivers {
+		w := cmd.OutOrStdout()
+		for k := range registry.List() {
+			_, _ = fmt.Fprintln(w, k)
+		}
+		return true, nil
 	}
 	return false, nil
 }
@@ -430,7 +549,7 @@ func createAction(cmd *cobra.Command, args []string) error {
 	if len(inst.Errors) > 0 {
 		return fmt.Errorf("errors inspecting instance: %+v", inst.Errors)
 	}
-	if _, err = instance.Prepare(cmd.Context(), inst); err != nil {
+	if _, err = instance.Prepare(cmd.Context(), inst, ""); err != nil {
 		return err
 	}
 	logrus.Infof("Run `limactl start %s` to start the instance.", inst.Name)
@@ -451,20 +570,25 @@ func startAction(cmd *cobra.Command, args []string) error {
 		return fmt.Errorf("errors inspecting instance: %+v", inst.Errors)
 	}
 	switch inst.Status {
-	case store.StatusRunning:
+	case limatype.StatusRunning:
 		logrus.Infof("The instance %q is already running. Run `%s` to open the shell.",
 			inst.Name, instance.LimactlShellCmd(inst.Name))
 		// Not an error
 		return nil
-	case store.StatusStopped:
+	case limatype.StatusStopped:
 		// NOP
 	default:
-		logrus.Warnf("expected status %q, got %q", store.StatusStopped, inst.Status)
+		logrus.Warnf("expected status %q, got %q", limatype.StatusStopped, inst.Status)
 	}
 	ctx := cmd.Context()
-	err = networks.Reconcile(ctx, inst.Name)
-	if err != nil {
-		return err
+	// Network reconciliation will be performed by the process launched by the autostart manager
+	if registered, err := autostart.IsRegistered(ctx, inst); err != nil && !errors.Is(err, autostart.ErrNotSupported) {
+		return fmt.Errorf("failed to check if the autostart entry for instance %q is registered: %w", inst.Name, err)
+	} else if (registered && autostart.AutoStartedIdentifier() != "") || !registered {
+		err = reconcile.Reconcile(ctx, inst.Name)
+		if err != nil {
+			return err
+		}
 	}
 
 	launchHostAgentForeground := false
@@ -483,7 +607,12 @@ func startAction(cmd *cobra.Command, args []string) error {
 		ctx = instance.WithWatchHostAgentTimeout(ctx, timeout)
 	}
 
-	return instance.Start(ctx, inst, "", launchHostAgentForeground)
+	progress, err := cmd.Flags().GetBool("progress")
+	if err != nil {
+		return err
+	}
+
+	return instance.Start(ctx, inst, launchHostAgentForeground, progress)
 }
 
 func createBashComplete(cmd *cobra.Command, _ []string, toComplete string) ([]string, cobra.ShellCompDirective) {
