@@ -4,6 +4,7 @@
 package main
 
 import (
+	"context"
 	"errors"
 	"fmt"
 	"os"
@@ -12,23 +13,22 @@ import (
 	"github.com/sirupsen/logrus"
 	"github.com/spf13/cobra"
 
-	"github.com/lima-vm/lima/pkg/limatmpl"
-	"github.com/lima-vm/lima/pkg/limayaml"
-	"github.com/lima-vm/lima/pkg/store/dirnames"
-	"github.com/lima-vm/lima/pkg/yqutil"
+	"github.com/lima-vm/lima/v2/pkg/driverutil"
+	"github.com/lima-vm/lima/v2/pkg/limatmpl"
+	"github.com/lima-vm/lima/v2/pkg/limatype/dirnames"
+	"github.com/lima-vm/lima/v2/pkg/limayaml"
+	"github.com/lima-vm/lima/v2/pkg/uiutil"
+	"github.com/lima-vm/lima/v2/pkg/yqutil"
 )
 
 func newTemplateCommand() *cobra.Command {
 	templateCommand := &cobra.Command{
 		Use:           "template",
 		Aliases:       []string{"tmpl"},
-		Short:         "Lima template management",
+		Short:         "Lima template management (EXPERIMENTAL)",
 		SilenceUsage:  true,
 		SilenceErrors: true,
 		GroupID:       advancedCommand,
-		// The template command is still hidden because the subcommands and options are still under development
-		// and subject to change at any time.
-		Hidden: true,
 		PreRun: func(*cobra.Command, []string) {
 			logrus.Warn("`limactl template` is experimental")
 		},
@@ -37,6 +37,7 @@ func newTemplateCommand() *cobra.Command {
 		newTemplateCopyCommand(),
 		newTemplateValidateCommand(),
 		newTemplateYQCommand(),
+		newTemplateURLCommand(),
 	)
 	return templateCommand
 }
@@ -48,13 +49,13 @@ func newValidateCommand() *cobra.Command {
 	return validateCommand
 }
 
-var templateCopyExample = `  Template locators are local files, file://, https://, or template:// URLs
+var templateCopyExample = `  Template locators are local files, file://, https://, or template: URLs
 
   # Copy default template to STDOUT
-  limactl template copy template://default -
+  limactl template copy template:default -
 
   # Copy template from web location to local file and embed all external references
-  # (this does not embed template:// references)
+  # (this does not embed template: references)
   limactl template copy --embed https://example.com/lima.yaml mighty-machine.yaml
 `
 
@@ -74,7 +75,7 @@ func newTemplateCopyCommand() *cobra.Command {
 	return templateCopyCommand
 }
 
-func fillDefaults(tmpl *limatmpl.Template) error {
+func fillDefaults(ctx context.Context, tmpl *limatmpl.Template) error {
 	limaDir, err := dirnames.LimaDir()
 	if err != nil {
 		return err
@@ -82,14 +83,19 @@ func fillDefaults(tmpl *limatmpl.Template) error {
 	// Load() will merge the template with override.yaml and default.yaml via FillDefaults().
 	// FillDefaults() needs the potential instance directory to validate host templates using {{.Dir}}.
 	filePath := filepath.Join(limaDir, tmpl.Name+".yaml")
-	tmpl.Config, err = limayaml.Load(tmpl.Bytes, filePath)
+	tmpl.Config, err = limayaml.Load(ctx, tmpl.Bytes, filePath)
 	if err == nil {
 		tmpl.Bytes, err = limayaml.Marshal(tmpl.Config, false)
 	}
+	if err := driverutil.ResolveVMType(ctx, tmpl.Config, filePath); err != nil {
+		logrus.Warnf("failed to resolve VM type for %q: %v", filePath, err)
+		return nil
+	}
 	return err
 }
 
 func templateCopyAction(cmd *cobra.Command, args []string) error {
+	ctx := cmd.Context()
 	source := args[0]
 	target := args[1]
 	embed, err := cmd.Flags().GetBool("embed")
@@ -137,10 +143,19 @@ func templateCopyAction(cmd *cobra.Command, args []string) error {
 		}
 	}
 	if fill {
-		if err := fillDefaults(tmpl); err != nil {
+		if err := fillDefaults(ctx, tmpl); err != nil {
 			return err
 		}
 	}
+	if target == "-" && uiutil.OutputIsTTY(cmd.OutOrStdout()) {
+		// run the output through YQ to colorize it
+		out, err := yqutil.EvaluateExpressionPlain(".", string(tmpl.Bytes), true)
+		if err == nil {
+			_, err = fmt.Fprint(cmd.OutOrStdout(), out)
+		}
+		return err
+	}
+
 	writer := cmd.OutOrStdout()
 	if target != "-" {
 		file, err := os.OpenFile(target, os.O_WRONLY|os.O_CREATE|os.O_TRUNC, 0o644)
@@ -159,10 +174,10 @@ External references are embedded and default values are filled in
 before the YQ expression is evaluated.
 
 Example:
-  limactl template yq template://default '.images[].location'
+  limactl template yq template:default '.images[].location'
 
 The example command is equivalent to using an external yq command like this:
-  limactl template copy --fill template://default - | yq '.images[].location'
+  limactl template copy --fill template:default - | yq '.images[].location'
 `
 
 func newTemplateYQCommand() *cobra.Command {
@@ -177,6 +192,7 @@ func newTemplateYQCommand() *cobra.Command {
 }
 
 func templateYQAction(cmd *cobra.Command, args []string) error {
+	ctx := cmd.Context()
 	locator := args[0]
 	expr := args[1]
 	tmpl, err := limatmpl.Read(cmd.Context(), "", locator)
@@ -189,10 +205,11 @@ func templateYQAction(cmd *cobra.Command, args []string) error {
 	if err := tmpl.Embed(cmd.Context(), true, true); err != nil {
 		return err
 	}
-	if err := fillDefaults(tmpl); err != nil {
+	if err := fillDefaults(ctx, tmpl); err != nil {
 		return err
 	}
-	out, err := yqutil.EvaluateExpressionPlain(expr, string(tmpl.Bytes))
+	colorsEnabled := uiutil.OutputIsTTY(cmd.OutOrStdout())
+	out, err := yqutil.EvaluateExpressionPlain(expr, string(tmpl.Bytes), colorsEnabled)
 	if err == nil {
 		_, err = fmt.Fprint(cmd.OutOrStdout(), out)
 	}
@@ -211,6 +228,7 @@ func newTemplateValidateCommand() *cobra.Command {
 }
 
 func templateValidateAction(cmd *cobra.Command, args []string) error {
+	ctx := cmd.Context()
 	fill, err := cmd.Flags().GetBool("fill")
 	if err != nil {
 		return err
@@ -238,10 +256,14 @@ func templateValidateAction(cmd *cobra.Command, args []string) error {
 		// Load() will merge the template with override.yaml and default.yaml via FillDefaults().
 		// FillDefaults() needs the potential instance directory to validate host templates using {{.Dir}}.
 		filePath := filepath.Join(limaDir, tmpl.Name+".yaml")
-		y, err := limayaml.Load(tmpl.Bytes, filePath)
+		y, err := limayaml.Load(ctx, tmpl.Bytes, filePath)
 		if err != nil {
 			return err
 		}
+		if err := driverutil.ResolveVMType(ctx, y, filePath); err != nil {
+			logrus.Warnf("failed to resolve VM type for %q: %v", filePath, err)
+			continue
+		}
 		if err := limayaml.Validate(y, false); err != nil {
 			return fmt.Errorf("failed to validate YAML file %q: %w", arg, err)
 		}
@@ -257,3 +279,22 @@ func templateValidateAction(cmd *cobra.Command, args []string) error {
 
 	return nil
 }
+
+func newTemplateURLCommand() *cobra.Command {
+	templateURLCommand := &cobra.Command{
+		Use:   "url CUSTOM_URL",
+		Short: "Transform custom template URLs to regular file or https URLs",
+		Args:  WrapArgsError(cobra.ExactArgs(1)),
+		RunE:  templateURLAction,
+	}
+	return templateURLCommand
+}
+
+func templateURLAction(cmd *cobra.Command, args []string) error {
+	url, err := limatmpl.TransformCustomURL(cmd.Context(), args[0])
+	if err != nil {
+		return err
+	}
+	_, err = fmt.Fprintln(cmd.OutOrStdout(), url)
+	return err
+}
