@@ -24,10 +24,10 @@ import (
 	"github.com/opencontainers/go-digest"
 	"github.com/sirupsen/logrus"
 
-	"github.com/lima-vm/lima/pkg/httpclientutil"
-	"github.com/lima-vm/lima/pkg/localpathutil"
-	"github.com/lima-vm/lima/pkg/lockutil"
-	"github.com/lima-vm/lima/pkg/progressbar"
+	"github.com/lima-vm/lima/v2/pkg/httpclientutil"
+	"github.com/lima-vm/lima/v2/pkg/localpathutil"
+	"github.com/lima-vm/lima/v2/pkg/lockutil"
+	"github.com/lima-vm/lima/v2/pkg/progressbar"
 )
 
 // HideProgress is used only for testing.
@@ -122,9 +122,6 @@ func WithDecompress(decompress bool) Opt {
 func WithExpectedDigest(expectedDigest digest.Digest) Opt {
 	return func(o *options) error {
 		if expectedDigest != "" {
-			if !expectedDigest.Algorithm().Available() {
-				return fmt.Errorf("expected digest algorithm %q is not available", expectedDigest.Algorithm())
-			}
 			if err := expectedDigest.Validate(); err != nil {
 				return err
 			}
@@ -567,8 +564,9 @@ func decompressLocal(ctx context.Context, decompressCmd, dst, src, ext, descript
 	bar.Start()
 	err = cmd.Run()
 	if err != nil {
-		if ee, ok := err.(*exec.ExitError); ok {
-			ee.Stderr = buf.Bytes()
+		var exitErr *exec.ExitError
+		if errors.As(err, &exitErr) {
+			exitErr.Stderr = buf.Bytes()
 		}
 	}
 	bar.Finish()
