@@ -16,8 +16,10 @@ import (
 	"path/filepath"
 	"regexp"
 	"runtime"
+	"slices"
 	"strconv"
 	"strings"
+	"sync"
 	"time"
 
 	"github.com/coreos/go-semver/semver"
@@ -27,21 +29,22 @@ import (
 	"github.com/mattn/go-shellwords"
 	"github.com/sirupsen/logrus"
 
-	"github.com/lima-vm/lima/pkg/fileutils"
-	"github.com/lima-vm/lima/pkg/iso9660util"
-	"github.com/lima-vm/lima/pkg/limayaml"
-	"github.com/lima-vm/lima/pkg/networks"
-	"github.com/lima-vm/lima/pkg/networks/usernet"
-	"github.com/lima-vm/lima/pkg/osutil"
-	"github.com/lima-vm/lima/pkg/qemuimgutil"
-	"github.com/lima-vm/lima/pkg/store"
-	"github.com/lima-vm/lima/pkg/store/filenames"
+	"github.com/lima-vm/lima/v2/pkg/fileutils"
+	"github.com/lima-vm/lima/v2/pkg/iso9660util"
+	"github.com/lima-vm/lima/v2/pkg/limatype"
+	"github.com/lima-vm/lima/v2/pkg/limatype/filenames"
+	"github.com/lima-vm/lima/v2/pkg/limayaml"
+	"github.com/lima-vm/lima/v2/pkg/networks"
+	"github.com/lima-vm/lima/v2/pkg/networks/usernet"
+	"github.com/lima-vm/lima/v2/pkg/osutil"
+	"github.com/lima-vm/lima/v2/pkg/qemuimgutil"
+	"github.com/lima-vm/lima/v2/pkg/store"
 )
 
 type Config struct {
 	Name         string
 	InstanceDir  string
-	LimaYAML     *limayaml.LimaYAML
+	LimaYAML     *limatype.LimaYAML
 	SSHLocalPort int
 	SSHAddress   string
 	VirtioGA     bool
@@ -56,7 +59,7 @@ type Config struct {
 // softMin must be >= hardMin.
 //
 // When updating this function, make sure to update
-// `website/content/en/docs/config/vmtype.md` too.
+// `website/content/en/docs/config/vmtype/qemu.md` too.
 func minimumQemuVersion() (hardMin, softMin semver.Version) {
 	var h, s string
 	switch runtime.GOOS {
@@ -85,7 +88,7 @@ func minimumQemuVersion() (hardMin, softMin semver.Version) {
 }
 
 // EnsureDisk also ensures the kernel and the initrd.
-func EnsureDisk(_ context.Context, cfg Config) error {
+func EnsureDisk(ctx context.Context, cfg Config) error {
 	diffDisk := filepath.Join(cfg.InstanceDir, filenames.DiffDisk)
 	if _, err := os.Stat(diffDisk); err == nil || !errors.Is(err, os.ErrNotExist) {
 		// disk is already ensured
@@ -102,7 +105,7 @@ func EnsureDisk(_ context.Context, cfg Config) error {
 	if err != nil {
 		return err
 	}
-	baseDiskInfo, err := qemuimgutil.GetInfo(baseDisk)
+	baseDiskInfo, err := qemuimgutil.GetInfo(ctx, baseDisk)
 	if err != nil {
 		return fmt.Errorf("failed to get the information of base disk %q: %w", baseDisk, err)
 	}
@@ -117,7 +120,7 @@ func EnsureDisk(_ context.Context, cfg Config) error {
 		args = append(args, "-F", baseDiskInfo.Format, "-b", baseDisk)
 	}
 	args = append(args, diffDisk, strconv.Itoa(int(diskSize)))
-	cmd := exec.Command("qemu-img", args...)
+	cmd := exec.CommandContext(ctx, "qemu-img", args...)
 	if out, err := cmd.CombinedOutput(); err != nil {
 		return fmt.Errorf("failed to run %v: %q: %w", cmd.Args, string(out), err)
 	}
@@ -148,11 +151,11 @@ func sendHmpCommand(cfg Config, cmd, tag string) (string, error) {
 	return rawClient.HumanMonitorCommand(hmc, nil)
 }
 
-func execImgCommand(cfg Config, args ...string) (string, error) {
+func execImgCommand(ctx context.Context, cfg Config, args ...string) (string, error) {
 	diffDisk := filepath.Join(cfg.InstanceDir, filenames.DiffDisk)
 	args = append(args, diffDisk)
 	logrus.Debugf("Running qemu-img %v command", args)
-	cmd := exec.Command("qemu-img", args...)
+	cmd := exec.CommandContext(ctx, "qemu-img", args...)
 	b, err := cmd.Output()
 	if err != nil {
 		return "", err
@@ -160,7 +163,7 @@ func execImgCommand(cfg Config, args ...string) (string, error) {
 	return string(b), err
 }
 
-func Del(cfg Config, run bool, tag string) error {
+func Del(ctx context.Context, cfg Config, run bool, tag string) error {
 	if run {
 		out, err := sendHmpCommand(cfg, "delvm", tag)
 		// there can still be output, even if no error!
@@ -170,11 +173,11 @@ func Del(cfg Config, run bool, tag string) error {
 		return err
 	}
 	// -d  deletes a snapshot
-	_, err := execImgCommand(cfg, "snapshot", "-d", tag)
+	_, err := execImgCommand(ctx, cfg, "snapshot", "-d", tag)
 	return err
 }
 
-func Save(cfg Config, run bool, tag string) error {
+func Save(ctx context.Context, cfg Config, run bool, tag string) error {
 	if run {
 		out, err := sendHmpCommand(cfg, "savevm", tag)
 		// there can still be output, even if no error!
@@ -184,11 +187,11 @@ func Save(cfg Config, run bool, tag string) error {
 		return err
 	}
 	// -c  creates a snapshot
-	_, err := execImgCommand(cfg, "snapshot", "-c", tag)
+	_, err := execImgCommand(ctx, cfg, "snapshot", "-c", tag)
 	return err
 }
 
-func Load(cfg Config, run bool, tag string) error {
+func Load(ctx context.Context, cfg Config, run bool, tag string) error {
 	if run {
 		out, err := sendHmpCommand(cfg, "loadvm", tag)
 		// there can still be output, even if no error!
@@ -198,12 +201,12 @@ func Load(cfg Config, run bool, tag string) error {
 		return err
 	}
 	// -a  applies a snapshot
-	_, err := execImgCommand(cfg, "snapshot", "-a", tag)
+	_, err := execImgCommand(ctx, cfg, "snapshot", "-a", tag)
 	return err
 }
 
 // List returns a space-separated list of all snapshots, with header and newlines.
-func List(cfg Config, run bool) (string, error) {
+func List(ctx context.Context, cfg Config, run bool) (string, error) {
 	if run {
 		out, err := sendHmpCommand(cfg, "info", "snapshots")
 		if err == nil {
@@ -215,7 +218,7 @@ func List(cfg Config, run bool) (string, error) {
 	}
 	// -l  lists all snapshots
 	args := []string{"snapshot", "-l"}
-	out, err := execImgCommand(cfg, args...)
+	out, err := execImgCommand(ctx, cfg, args...)
 	if err == nil {
 		// remove the redundant heading, result is not machine-parseable
 		out = strings.Replace(out, "Snapshot list:\n", "", 1)
@@ -286,13 +289,13 @@ type features struct {
 	CPUHelp []byte
 }
 
-func inspectFeatures(exe, machine string) (*features, error) {
+func inspectFeatures(ctx context.Context, exe, machine string) (*features, error) {
 	var (
 		f      features
 		stdout bytes.Buffer
 		stderr bytes.Buffer
 	)
-	cmd := exec.Command(exe, "-M", "none", "-accel", "help")
+	cmd := exec.CommandContext(ctx, exe, "-M", "none", "-accel", "help")
 	cmd.Stdout = &stdout
 	cmd.Stderr = &stderr
 	if err := cmd.Run(); err != nil {
@@ -304,7 +307,7 @@ func inspectFeatures(exe, machine string) (*features, error) {
 		f.AccelHelp = stderr.Bytes()
 	}
 
-	cmd = exec.Command(exe, "-M", "none", "-netdev", "help")
+	cmd = exec.CommandContext(ctx, exe, "-M", "none", "-netdev", "help")
 	cmd.Stdout = &stdout
 	cmd.Stderr = &stderr
 	if err := cmd.Run(); err != nil {
@@ -316,7 +319,7 @@ func inspectFeatures(exe, machine string) (*features, error) {
 		}
 	}
 
-	cmd = exec.Command(exe, "-machine", "help")
+	cmd = exec.CommandContext(ctx, exe, "-machine", "help")
 	cmd.Stdout = &stdout
 	cmd.Stderr = &stderr
 	if err := cmd.Run(); err != nil {
@@ -329,7 +332,7 @@ func inspectFeatures(exe, machine string) (*features, error) {
 	}
 
 	// Avoid error: "No machine specified, and there is no default"
-	cmd = exec.Command(exe, "-cpu", "help", "-machine", machine)
+	cmd = exec.CommandContext(ctx, exe, "-cpu", "help", "-machine", machine)
 	cmd.Stdout = &stdout
 	cmd.Stderr = &stderr
 	if err := cmd.Run(); err != nil {
@@ -383,8 +386,8 @@ func adjustMemBytesDarwinARM64HVF(memBytes int64, accel string) int64 {
 }
 
 // qemuMachine returns string to use for -machine.
-func qemuMachine(arch limayaml.Arch) string {
-	if arch == limayaml.X8664 {
+func qemuMachine(arch limatype.Arch) string {
+	if arch == limatype.X8664 {
 		return "q35"
 	}
 	return "virt"
@@ -403,6 +406,75 @@ func audioDevice() string {
 	return "oss"
 }
 
+func defaultCPUType() limatype.CPUType {
+	// x86_64 + TCG + max was previously unstable until 2021.
+	// https://bugzilla.redhat.com/show_bug.cgi?id=1999700
+	// https://bugs.launchpad.net/qemu/+bug/1748296
+	defaultX8664 := "max"
+	if runtime.GOOS == "windows" && runtime.GOARCH == "amd64" {
+		// https://github.com/lima-vm/lima/pull/3487#issuecomment-2846253560
+		// > #931 intentionally prevented the code from setting it to max when running on Windows,
+		// > and kept it at qemu64.
+		//
+		// TODO: remove this if "max" works with the latest qemu
+		defaultX8664 = "qemu64"
+	}
+	cpuType := map[limatype.Arch]string{
+		limatype.AARCH64: "max",
+		limatype.ARMV7L:  "max",
+		limatype.X8664:   defaultX8664,
+		limatype.PPC64LE: "max",
+		limatype.RISCV64: "max",
+		limatype.S390X:   "max",
+	}
+	for arch := range cpuType {
+		if limayaml.IsNativeArch(arch) && Accel(arch) != "tcg" {
+			if hasHostCPU() {
+				cpuType[arch] = "host"
+			}
+		}
+		if arch == limatype.X8664 && runtime.GOOS == "darwin" {
+			// disable AVX-512, since it requires trapping instruction faults in guest
+			// Enterprise Linux requires either v2 (SSE4) or v3 (AVX2), but not yet v4.
+			cpuType[arch] += ",-avx512vl"
+
+			// Disable pdpe1gb on Intel Mac
+			// https://github.com/lima-vm/lima/issues/1485
+			// https://stackoverflow.com/a/72863744/5167443
+			cpuType[arch] += ",-pdpe1gb"
+		}
+	}
+	return cpuType
+}
+
+func resolveCPUType(y *limatype.LimaYAML) string {
+	cpuType := defaultCPUType()
+	var overrideCPUType bool
+	var qemuOpts limatype.QEMUOpts
+	if err := limayaml.Convert(y.VMOpts[limatype.QEMU], &qemuOpts, "vmOpts.qemu"); err != nil {
+		logrus.WithError(err).Warnf("Couldn't convert %q", y.VMOpts[limatype.QEMU])
+	}
+	for k, v := range qemuOpts.CPUType {
+		if !slices.Contains(limatype.ArchTypes, *y.Arch) {
+			logrus.Warnf("field `vmOpts.qemu.cpuType` uses unsupported arch %q", k)
+			continue
+		}
+		if v != "" {
+			overrideCPUType = true
+			cpuType[k] = v
+		}
+	}
+	if overrideCPUType {
+		qemuOpts.CPUType = cpuType
+		if y.VMOpts == nil {
+			y.VMOpts = limatype.VMOpts{}
+		}
+		y.VMOpts[limatype.QEMU] = qemuOpts
+	}
+
+	return cpuType[*y.Arch]
+}
+
 func Cmdline(ctx context.Context, cfg Config) (exe string, args []string, err error) {
 	y := cfg.LimaYAML
 	exe, args, err = Exe(*y.Arch)
@@ -410,12 +482,12 @@ func Cmdline(ctx context.Context, cfg Config) (exe string, args []string, err er
 		return "", nil, err
 	}
 
-	features, err := inspectFeatures(exe, qemuMachine(*y.Arch))
+	features, err := inspectFeatures(ctx, exe, qemuMachine(*y.Arch))
 	if err != nil {
 		return "", nil, err
 	}
 
-	version, err := getQemuVersion(exe)
+	version, err := getQemuVersion(ctx, exe)
 	if err != nil {
 		logrus.WithError(err).Warning("Failed to detect QEMU version")
 	} else {
@@ -427,8 +499,12 @@ func Cmdline(ctx context.Context, cfg Config) (exe string, args []string, err er
 		if version.LessThan(softMin) {
 			logrus.Warnf("QEMU %v is too old, %v or later is recommended", version, softMin)
 		}
-		if y.VMOpts.QEMU.MinimumVersion != nil && version.LessThan(*semver.New(*y.VMOpts.QEMU.MinimumVersion)) {
-			logrus.Fatalf("QEMU %v is too old, template requires %q or later", version, *y.VMOpts.QEMU.MinimumVersion)
+		var qemuOpts limatype.QEMUOpts
+		if err := limayaml.Convert(y.VMOpts[limatype.QEMU], &qemuOpts, "vmOpts.qemu"); err != nil {
+			logrus.WithError(err).Warnf("Couldn't convert %q", y.VMOpts[limatype.QEMU])
+		}
+		if qemuOpts.MinimumVersion != nil && version.LessThan(*semver.New(*qemuOpts.MinimumVersion)) {
+			logrus.Fatalf("QEMU %v is too old, template requires %q or later", version, *qemuOpts.MinimumVersion)
 		}
 	}
 
@@ -446,14 +522,14 @@ func Cmdline(ctx context.Context, cfg Config) (exe string, args []string, err er
 	memBytes = adjustMemBytesDarwinARM64HVF(memBytes, accel)
 	args = appendArgsIfNoConflict(args, "-m", strconv.Itoa(int(memBytes>>20)))
 
-	if *y.MountType == limayaml.VIRTIOFS {
+	if *y.MountType == limatype.VIRTIOFS {
 		args = appendArgsIfNoConflict(args, "-object",
 			fmt.Sprintf("memory-backend-file,id=virtiofs-shm,size=%s,mem-path=/dev/shm,share=on", strconv.Itoa(int(memBytes))))
 		args = appendArgsIfNoConflict(args, "-numa", "node,memdev=virtiofs-shm")
 	}
 
 	// CPU
-	cpu := y.CPUType[*y.Arch]
+	cpu := resolveCPUType(y)
 	if runtime.GOOS == "darwin" && runtime.GOARCH == "amd64" {
 		switch {
 		case strings.HasPrefix(cpu, "host"), strings.HasPrefix(cpu, "max"):
@@ -470,7 +546,7 @@ func Cmdline(ctx context.Context, cfg Config) (exe string, args []string, err er
 
 	// Machine
 	switch *y.Arch {
-	case limayaml.X8664:
+	case limatype.X8664:
 		switch accel {
 		case "tcg":
 			// use q35 machine with vmware io port disabled.
@@ -489,10 +565,10 @@ func Cmdline(ctx context.Context, cfg Config) (exe string, args []string, err er
 		default:
 			args = appendArgsIfNoConflict(args, "-machine", "q35,accel="+accel)
 		}
-	case limayaml.AARCH64:
+	case limatype.AARCH64:
 		machine := "virt,accel=" + accel
 		args = appendArgsIfNoConflict(args, "-machine", machine)
-	case limayaml.RISCV64:
+	case limatype.RISCV64:
 		// https://github.com/tianocore/edk2/blob/edk2-stable202408/OvmfPkg/RiscVVirt/README.md#test
 		// > Note: the `acpi=off` machine property is specified because Linux guest
 		// > support for ACPI (that is, the ACPI consumer side) is a work in progress.
@@ -500,13 +576,13 @@ func Cmdline(ctx context.Context, cfg Config) (exe string, args []string, err er
 		// > yourself.
 		machine := "virt,acpi=off,accel=" + accel
 		args = appendArgsIfNoConflict(args, "-machine", machine)
-	case limayaml.ARMV7L:
+	case limatype.ARMV7L:
 		machine := "virt,accel=" + accel
 		args = appendArgsIfNoConflict(args, "-machine", machine)
-	case limayaml.PPC64LE:
+	case limatype.PPC64LE:
 		machine := "pseries,accel=" + accel
 		args = appendArgsIfNoConflict(args, "-machine", machine)
-	case limayaml.S390X:
+	case limatype.S390X:
 		machine := "s390-ccw-virtio,accel=" + accel
 		args = appendArgsIfNoConflict(args, "-machine", machine)
 	}
@@ -517,11 +593,11 @@ func Cmdline(ctx context.Context, cfg Config) (exe string, args []string, err er
 
 	// Firmware
 	legacyBIOS := *y.Firmware.LegacyBIOS
-	if legacyBIOS && *y.Arch != limayaml.X8664 && *y.Arch != limayaml.ARMV7L {
+	if legacyBIOS && *y.Arch != limatype.X8664 && *y.Arch != limatype.ARMV7L {
 		logrus.Warnf("field `firmware.legacyBIOS` is not supported for architecture %q, ignoring", *y.Arch)
 		legacyBIOS = false
 	}
-	noFirmware := *y.Arch == limayaml.PPC64LE || *y.Arch == limayaml.S390X || legacyBIOS
+	noFirmware := *y.Arch == limatype.PPC64LE || *y.Arch == limatype.S390X || legacyBIOS
 	if !noFirmware {
 		var firmware string
 		firmwareInBios := runtime.GOOS == "windows"
@@ -533,7 +609,7 @@ func Cmdline(ctx context.Context, cfg Config) (exe string, args []string, err er
 				firmwareInBios = b
 			}
 		}
-		firmwareInBios = firmwareInBios && *y.Arch == limayaml.X8664
+		firmwareInBios = firmwareInBios && *y.Arch == limatype.X8664
 		downloadedFirmware := filepath.Join(cfg.InstanceDir, filenames.QemuEfiCodeFD)
 		firmwareWithVars := filepath.Join(cfg.InstanceDir, filenames.QemuEfiFullFD)
 		if firmwareInBios {
@@ -546,7 +622,7 @@ func Cmdline(ctx context.Context, cfg Config) (exe string, args []string, err er
 			loop:
 				for _, f := range y.Firmware.Images {
 					switch f.VMType {
-					case "", limayaml.QEMU:
+					case "", limatype.QEMU:
 						if f.Arch == *y.Arch {
 							if _, err = fileutils.DownloadFile(ctx, downloadedFirmware, f.File, true, "UEFI code "+f.Location, *y.Arch); err != nil {
 								logrus.WithError(err).Warnf("failed to download %q", f.Location)
@@ -656,7 +732,7 @@ func Cmdline(ctx context.Context, cfg Config) (exe string, args []string, err er
 	if diskSize, _ := units.RAMInBytes(*cfg.LimaYAML.Disk); diskSize > 0 {
 		args = append(args, "-drive", fmt.Sprintf("file=%s,if=virtio,discard=on", diffDisk))
 	} else if !isBaseDiskCDROM {
-		baseDiskInfo, err := qemuimgutil.GetInfo(baseDisk)
+		baseDiskInfo, err := qemuimgutil.GetInfo(ctx, baseDisk)
 		if err != nil {
 			return "", nil, fmt.Errorf("failed to get the information of %q: %w", baseDisk, err)
 		}
@@ -706,7 +782,7 @@ func Cmdline(ctx context.Context, cfg Config) (exe string, args []string, err er
 		args = append(args, "-netdev", fmt.Sprintf("socket,id=net0,fd={{ fd_connect %q }}", qemuSock))
 	}
 	virtioNet := "virtio-net-pci"
-	if *y.Arch == limayaml.S390X {
+	if *y.Arch == limatype.S390X {
 		// virtio-net-pci does not work on EL, while it works on Ubuntu
 		// https://github.com/lima-vm/lima/pull/3319/files#r1986388345
 		virtioNet = "virtio-net-ccw"
@@ -789,16 +865,18 @@ func Cmdline(ctx context.Context, cfg Config) (exe string, args []string, err er
 		args = appendArgsIfNoConflict(args, "-display", display)
 	}
 
-	switch *y.Arch {
-	// FIXME: use virtio-gpu on all the architectures
-	case limayaml.X8664, limayaml.RISCV64:
-		args = append(args, "-device", "virtio-vga")
-	default:
-		args = append(args, "-device", "virtio-gpu")
+	if *y.Video.Display != "none" {
+		switch *y.Arch {
+		// FIXME: use virtio-gpu on all the architectures
+		case limatype.X8664, limatype.RISCV64:
+			args = append(args, "-device", "virtio-vga")
+		default:
+			args = append(args, "-device", "virtio-gpu")
+		}
+		args = append(args, "-device", "virtio-keyboard-pci")
+		args = append(args, "-device", "virtio-"+input+"-pci")
+		args = append(args, "-device", "qemu-xhci,id=usb-bus")
 	}
-	args = append(args, "-device", "virtio-keyboard-pci")
-	args = append(args, "-device", "virtio-"+input+"-pci")
-	args = append(args, "-device", "qemu-xhci,id=usb-bus")
 
 	// Parallel
 	args = append(args, "-parallel", "none")
@@ -821,7 +899,7 @@ func Cmdline(ctx context.Context, cfg Config) (exe string, args []string, err er
 	// On ARM, the default serial is ttyAMA0, this PCI serial is ttyS0.
 	// https://gitlab.com/qemu-project/qemu/-/issues/1801#note_1494720586
 	switch *y.Arch {
-	case limayaml.AARCH64, limayaml.ARMV7L:
+	case limatype.AARCH64, limatype.ARMV7L:
 		serialpSock := filepath.Join(cfg.InstanceDir, filenames.SerialPCISock)
 		if err := os.RemoveAll(serialpSock); err != nil {
 			return "", nil, err
@@ -848,7 +926,7 @@ func Cmdline(ctx context.Context, cfg Config) (exe string, args []string, err er
 	args = append(args, "-chardev", fmt.Sprintf("socket,id=%s,path=%s,server=on,wait=off,logfile=%s", serialvChardev, serialvSock, serialvLog))
 	// max_ports=1 is required for https://github.com/lima-vm/lima/issues/1689 https://github.com/lima-vm/lima/issues/1691
 	serialvMaxPorts := 1
-	if *y.Arch == limayaml.S390X {
+	if *y.Arch == limatype.S390X {
 		serialvMaxPorts++ // needed to avoid `virtio-serial-bus: Out-of-range port id specified, max. allowed: 0`
 	}
 	args = append(args, "-device", fmt.Sprintf("virtio-serial-pci,id=virtio-serial0,max_ports=%d", serialvMaxPorts))
@@ -856,7 +934,7 @@ func Cmdline(ctx context.Context, cfg Config) (exe string, args []string, err er
 
 	// We also want to enable vsock here, but QEMU does not support vsock for macOS hosts
 
-	if *y.MountType == limayaml.NINEP || *y.MountType == limayaml.VIRTIOFS {
+	if *y.MountType == limatype.NINEP || *y.MountType == limatype.VIRTIOFS {
 		for i, f := range y.Mounts {
 			tag := fmt.Sprintf("mount%d", i)
 			if err := os.MkdirAll(f.Location, 0o755); err != nil {
@@ -864,16 +942,16 @@ func Cmdline(ctx context.Context, cfg Config) (exe string, args []string, err er
 			}
 
 			switch *y.MountType {
-			case limayaml.NINEP:
+			case limatype.NINEP:
 				options := "local"
 				options += fmt.Sprintf(",mount_tag=%s", tag)
 				options += fmt.Sprintf(",path=%s", f.Location)
 				options += fmt.Sprintf(",security_model=%s", *f.NineP.SecurityModel)
 				if !*f.Writable {
-					options += ",readonly"
+					options += ",readonly=on"
 				}
 				args = append(args, "-virtfs", options)
-			case limayaml.VIRTIOFS:
+			case limatype.VIRTIOFS:
 				// Note that read-only mode is not supported on the QEMU/virtiofsd side yet:
 				// https://gitlab.com/virtio-fs/virtiofsd/-/issues/97
 				chardev := fmt.Sprintf("char-virtiofs-%d", i)
@@ -913,7 +991,7 @@ func Cmdline(ctx context.Context, cfg Config) (exe string, args []string, err er
 	return exe, args, nil
 }
 
-func FindVirtiofsd(qemuExe string) (string, error) {
+func FindVirtiofsd(ctx context.Context, qemuExe string) (string, error) {
 	type vhostUserBackend struct {
 		BackendType string `json:"type"`
 		Binary      string `json:"binary"`
@@ -972,7 +1050,7 @@ func FindVirtiofsd(qemuExe string) (string, error) {
 
 			// Only rust virtiofsd supports --version, so use that to make sure this isn't
 			// QEMU's virtiofsd, which requires running as root.
-			cmd := exec.Command(vhostCfg.Binary, "--version")
+			cmd := exec.CommandContext(ctx, vhostCfg.Binary, "--version")
 			output, err := cmd.CombinedOutput()
 			if err != nil {
 				logrus.Warnf("Failed to run %s --version (is this QEMU virtiofsd?): %s: %s",
@@ -1003,11 +1081,11 @@ func VirtiofsdCmdline(cfg Config, mountIndex int) ([]string, error) {
 }
 
 // qemuArch returns the arch string used by qemu.
-func qemuArch(arch limayaml.Arch) string {
+func qemuArch(arch limatype.Arch) string {
 	switch arch {
-	case limayaml.ARMV7L:
+	case limatype.ARMV7L:
 		return "arm"
-	case limayaml.PPC64LE:
+	case limatype.PPC64LE:
 		return "ppc64"
 	default:
 		return arch
@@ -1015,14 +1093,14 @@ func qemuArch(arch limayaml.Arch) string {
 }
 
 // qemuEdk2 returns the arch string used by `/usr/local/share/qemu/edk2-*-code.fd`.
-func qemuEdk2Arch(arch limayaml.Arch) string {
-	if arch == limayaml.RISCV64 {
+func qemuEdk2Arch(arch limatype.Arch) string {
+	if arch == limatype.RISCV64 {
 		return "riscv"
 	}
 	return qemuArch(arch)
 }
 
-func Exe(arch limayaml.Arch) (exe string, args []string, err error) {
+func Exe(arch limatype.Arch) (exe string, args []string, err error) {
 	exeBase := "qemu-system-" + qemuArch(arch)
 	envK := "QEMU_SYSTEM_" + strings.ToUpper(qemuArch(arch))
 	if envV := os.Getenv(envK); envV != "" {
@@ -1042,12 +1120,17 @@ func Exe(arch limayaml.Arch) (exe string, args []string, err error) {
 	return exe, args, nil
 }
 
-func Accel(arch limayaml.Arch) string {
+func Accel(arch limatype.Arch) string {
 	if limayaml.IsNativeArch(arch) {
 		switch runtime.GOOS {
 		case "darwin":
+			// TODO: return "tcg" if HVF is not available
 			return "hvf"
 		case "linux":
+			if _, err := os.Stat("/dev/kvm"); err != nil {
+				logrus.WithError(err).Warn("/dev/kvm is not available. Disabling KVM. Expect very poor performance.")
+				return "tcg"
+			}
 			return "kvm"
 		case "netbsd":
 			return "nvmm"
@@ -1070,12 +1153,12 @@ func parseQemuVersion(output string) (*semver.Version, error) {
 	return &semver.Version{}, fmt.Errorf("failed to parse %v", output)
 }
 
-func getQemuVersion(qemuExe string) (*semver.Version, error) {
+func getQemuVersion(ctx context.Context, qemuExe string) (*semver.Version, error) {
 	var (
 		stdout bytes.Buffer
 		stderr bytes.Buffer
 	)
-	cmd := exec.Command(qemuExe, "--version")
+	cmd := exec.CommandContext(ctx, qemuExe, "--version")
 	cmd.Stdout = &stdout
 	cmd.Stderr = &stderr
 	if err := cmd.Run(); err != nil {
@@ -1085,9 +1168,9 @@ func getQemuVersion(qemuExe string) (*semver.Version, error) {
 	return parseQemuVersion(stdout.String())
 }
 
-func getFirmware(qemuExe string, arch limayaml.Arch) (string, error) {
+func getFirmware(qemuExe string, arch limatype.Arch) (string, error) {
 	switch arch {
-	case limayaml.X8664, limayaml.AARCH64, limayaml.ARMV7L, limayaml.RISCV64:
+	case limatype.X8664, limatype.AARCH64, limatype.ARMV7L, limatype.RISCV64:
 	default:
 		return "", fmt.Errorf("unexpected architecture: %q", arch)
 	}
@@ -1110,7 +1193,7 @@ func getFirmware(qemuExe string, arch limayaml.Arch) (string, error) {
 	}
 
 	switch arch {
-	case limayaml.X8664:
+	case limatype.X8664:
 		// Archlinux package "edk2-ovmf"
 		// @see: https://archlinux.org/packages/extra/any/edk2-ovmf/files
 		candidates = append(candidates, "/usr/share/edk2/x64/OVMF_CODE.4m.fd")
@@ -1121,7 +1204,7 @@ func getFirmware(qemuExe string, arch limayaml.Arch) (string, error) {
 		candidates = append(candidates, "/usr/share/edk2/ovmf/OVMF_CODE.fd")
 		// openSUSE package "qemu-ovmf-x86_64"
 		candidates = append(candidates, "/usr/share/qemu/ovmf-x86_64.bin")
-	case limayaml.AARCH64:
+	case limatype.AARCH64:
 		// Archlinux package "edk2-aarch64"
 		// @see: https://archlinux.org/packages/extra/any/edk2-aarch64/files
 		candidates = append(candidates, "/usr/share/edk2/aarch64/QEMU_CODE.fd")
@@ -1130,14 +1213,14 @@ func getFirmware(qemuExe string, arch limayaml.Arch) (string, error) {
 		candidates = append(candidates, "/usr/share/AAVMF/AAVMF_CODE.fd")
 		// Debian package "qemu-efi-aarch64" (unpadded, backwards compatibility)
 		candidates = append(candidates, "/usr/share/qemu-efi-aarch64/QEMU_EFI.fd")
-	case limayaml.ARMV7L:
+	case limatype.ARMV7L:
 		// Archlinux package "edk2-arm"
 		// @see: https://archlinux.org/packages/extra/any/edk2-arm/files
 		candidates = append(candidates, "/usr/share/edk2/arm/QEMU_CODE.fd")
 		// Debian package "qemu-efi-arm"
 		// Fedora package "edk2-arm"
 		candidates = append(candidates, "/usr/share/AAVMF/AAVMF32_CODE.fd")
-	case limayaml.RISCV64:
+	case limatype.RISCV64:
 		// Debian package "qemu-efi-riscv64"
 		candidates = append(candidates, "/usr/share/qemu-efi-riscv64/RISCV_VIRT_CODE.fd")
 		// Fedora package "edk2-riscv64"
@@ -1152,17 +1235,17 @@ func getFirmware(qemuExe string, arch limayaml.Arch) (string, error) {
 		}
 	}
 
-	if arch == limayaml.X8664 {
+	if arch == limatype.X8664 {
 		return "", fmt.Errorf("could not find firmware for %q (hint: try setting `firmware.legacyBIOS` to `true`)", arch)
 	}
 	qemuArch := strings.TrimPrefix(filepath.Base(qemuExe), "qemu-system-")
 	return "", fmt.Errorf("could not find firmware for %q (hint: try copying the \"edk-%s-code.fd\" firmware to $HOME/.local/share/qemu/)", arch, qemuArch)
 }
 
-func getFirmwareVars(qemuExe string, arch limayaml.Arch) (string, error) {
+func getFirmwareVars(qemuExe string, arch limatype.Arch) (string, error) {
 	var targetArch string
 	switch arch {
-	case limayaml.X8664:
+	case limatype.X8664:
 		targetArch = "i386" // vars are unified between i386 and x86_64 and normally only former is bundled
 	default:
 		return "", fmt.Errorf("unexpected architecture: %q", arch)
@@ -1195,3 +1278,43 @@ func getFirmwareVars(qemuExe string, arch limayaml.Arch) (string, error) {
 
 	return "", fmt.Errorf("could not find firmware vars for %q", arch)
 }
+
+var hasSMEDarwin = sync.OnceValue(func() bool {
+	if runtime.GOOS != "darwin" || runtime.GOARCH != "arm64" {
+		return false
+	}
+	// golang.org/x/sys/cpu does not support inspecting the availability of SME yet
+	s, err := osutil.Sysctl(context.Background(), "hw.optional.arm.FEAT_SME")
+	if err != nil {
+		logrus.WithError(err).Debug("failed to check hw.optional.arm.FEAT_SME")
+	}
+	return s == "1"
+})
+
+func hasHostCPU() bool {
+	switch runtime.GOOS {
+	case "darwin":
+		if hasSMEDarwin() {
+			// [2025-02-05]
+			// SME is available since Apple M4 running macOS 15.2, but it was broken on macOS 15.2.
+			// It has been fixed in 15.3.
+			//
+			// https://github.com/lima-vm/lima/issues/3032
+			// https://gitlab.com/qemu-project/qemu/-/issues/2665
+			// https://gitlab.com/qemu-project/qemu/-/issues/2721
+
+			// [2025-02-12]
+			// SME got broken again after upgrading QEMU from 9.2.0 to 9.2.1 (Homebrew bottle).
+			// Possibly this regression happened in some build process rather than in QEMU itself?
+			// https://github.com/lima-vm/lima/issues/3226
+			return false
+		}
+		return true
+	case "linux":
+		return true
+	case "netbsd", "windows":
+		return false
+	}
+	// Not reached
+	return false
+}
