@@ -5,8 +5,9 @@ package guestagent
 
 import (
 	"context"
+	"io"
 
-	"github.com/lima-vm/lima/pkg/guestagent/api"
+	"github.com/lima-vm/lima/v2/pkg/guestagent/api"
 )
 
 type Agent interface {
@@ -14,4 +15,5 @@ type Agent interface {
 	Events(ctx context.Context, ch chan *api.Event)
 	LocalPorts(ctx context.Context) ([]*api.IPPort, error)
 	HandleInotify(event *api.Inotify)
+	io.Closer
 }
