@@ -25,7 +25,7 @@ func newFakeKubeClient() (clientSet.Interface, informers.SharedInformerFactory)
 }
 
 func TestGetPorts(t *testing.T) {
-	ctx, cancel := context.WithCancel(context.Background())
+	ctx, cancel := context.WithCancel(t.Context())
 	defer cancel()
 	serviceCreatedCh := make(chan struct{}, 1)
 	kubeClient, informerFactory := newFakeKubeClient()
@@ -58,13 +58,24 @@ func TestGetPorts(t *testing.T) {
 							TargetPort: intstr.FromInt(80),
 							NodePort:   8080,
 						},
+						{
+							Name:       "dns",
+							Protocol:   corev1.ProtocolUDP,
+							Port:       53,
+							TargetPort: intstr.FromInt(53),
+							NodePort:   5353,
+						},
 					},
 				},
 			},
 			want: []Entry{{
 				Protocol: TCP,
-				IP:       net.ParseIP("0.0.0.0"),
+				IP:       net.IPv4zero,
 				Port:     8080,
+			}, {
+				Protocol: UDP,
+				IP:       net.IPv4zero,
+				Port:     5353,
 			}},
 		},
 		{
@@ -85,7 +96,7 @@ func TestGetPorts(t *testing.T) {
 			},
 			want: []Entry{{
 				Protocol: TCP,
-				IP:       net.ParseIP("0.0.0.0"),
+				IP:       net.IPv4zero,
 				Port:     8081,
 			}},
 		},
