@@ -4,6 +4,7 @@
 package hostagent
 
 import (
+	"context"
 	"errors"
 	"fmt"
 	"os"
@@ -12,21 +13,22 @@ import (
 	"github.com/lima-vm/sshocker/pkg/reversesshfs"
 	"github.com/sirupsen/logrus"
 
-	"github.com/lima-vm/lima/pkg/ioutilx"
-	"github.com/lima-vm/lima/pkg/limayaml"
+	"github.com/lima-vm/lima/v2/pkg/ioutilx"
+	"github.com/lima-vm/lima/v2/pkg/limatype"
+	"github.com/lima-vm/lima/v2/pkg/sshutil"
 )
 
 type mount struct {
 	close func() error
 }
 
-func (a *HostAgent) setupMounts() ([]*mount, error) {
+func (a *HostAgent) setupMounts(ctx context.Context) ([]*mount, error) {
 	var (
 		res  []*mount
 		errs []error
 	)
 	for _, f := range a.instConfig.Mounts {
-		m, err := a.setupMount(f)
+		m, err := a.setupMount(ctx, f)
 		if err != nil {
 			errs = append(errs, err)
 			continue
@@ -36,7 +38,7 @@ func (a *HostAgent) setupMounts() ([]*mount, error) {
 	return res, errors.Join(errs...)
 }
 
-func (a *HostAgent) setupMount(m limayaml.Mount) (*mount, error) {
+func (a *HostAgent) setupMount(ctx context.Context, m limatype.Mount) (*mount, error) {
 	if err := os.MkdirAll(m.Location, 0o755); err != nil {
 		return nil, err
 	}
@@ -53,18 +55,22 @@ func (a *HostAgent) setupMount(m limayaml.Mount) (*mount, error) {
 	resolvedLocation := m.Location
 	if runtime.GOOS == "windows" {
 		var err error
-		resolvedLocation, err = ioutilx.WindowsSubsystemPath(m.Location)
+		resolvedLocation, err = ioutilx.WindowsSubsystemPath(ctx, m.Location)
 		if err != nil {
 			return nil, err
 		}
 	}
 
+	sshAddress, sshPort := a.sshAddressPort()
+	// Create a copy of sshConfig to avoid
+	// modifying HostAgent's sshConfig in case of Windows
+	sshConfig := *a.sshConfig
 	rsf := &reversesshfs.ReverseSSHFS{
 		Driver:              *m.SSHFS.SFTPDriver,
-		SSHConfig:           a.sshConfig,
+		SSHConfig:           &sshConfig,
 		LocalPath:           resolvedLocation,
-		Host:                "127.0.0.1",
-		Port:                a.sshLocalPort,
+		Host:                sshAddress,
+		Port:                sshPort,
 		RemotePath:          *m.MountPoint,
 		Readonly:            !(*m.Writable),
 		SSHFSAdditionalArgs: []string{"-o", sshfsOptions},
@@ -75,6 +81,10 @@ func (a *HostAgent) setupMount(m limayaml.Mount) (*mount, error) {
 		// 2. these errors still imply additional coms over mux socket, which resulted sftp-server to fail more often statistically during test runs.
 		// It is reasonable to disable this on Windows if required feature is not fully operational.
 		rsf.SSHConfig.Persist = false
+
+		// HostAgent's `sshConfig` already has some ControlMaster related args in `AdditionalArgs`,
+		// so it is necessary to remove them to avoid overrides above `Persist=false`.
+		rsf.SSHConfig.AdditionalArgs = sshutil.DisableControlMasterOptsFromSSHArgs(rsf.SSHConfig.AdditionalArgs)
 	}
 	if err := rsf.Prepare(); err != nil {
 		return nil, fmt.Errorf("failed to prepare reverse sshfs for %q on %q: %w", resolvedLocation, *m.MountPoint, err)
