@@ -14,7 +14,7 @@ import (
 	"github.com/mattn/go-isatty"
 	"github.com/sirupsen/logrus"
 
-	"github.com/lima-vm/lima/pkg/osutil"
+	"github.com/lima-vm/lima/v2/pkg/osutil"
 )
 
 func execHostAgentForeground(limactl string, haCmd *exec.Cmd) error {
