@@ -18,11 +18,12 @@ import (
 	"github.com/coreos/go-semver/semver"
 	"github.com/sirupsen/logrus"
 
-	"github.com/lima-vm/lima/pkg/limayaml"
-	"github.com/lima-vm/lima/pkg/store/dirnames"
-	"github.com/lima-vm/lima/pkg/store/filenames"
-	"github.com/lima-vm/lima/pkg/version/versionutil"
-	"github.com/lima-vm/lima/pkg/yqutil"
+	"github.com/lima-vm/lima/v2/pkg/limatype"
+	"github.com/lima-vm/lima/v2/pkg/limatype/dirnames"
+	"github.com/lima-vm/lima/v2/pkg/limatype/filenames"
+	"github.com/lima-vm/lima/v2/pkg/limayaml"
+	"github.com/lima-vm/lima/v2/pkg/version/versionutil"
+	"github.com/lima-vm/lima/v2/pkg/yqutil"
 )
 
 // Embed will recursively resolve all "base" dependencies and update the
@@ -76,7 +77,7 @@ func (tmpl *Template) embedAllBases(ctx context.Context, embedAll, defaultBase b
 		}
 		isTemplate, _ := SeemsTemplateURL(baseLocator.URL)
 		if isTemplate && !embedAll {
-			// Once we skip a template:// URL we can no longer embed any other base template
+			// Once we skip a template: URL we can no longer embed any other base template
 			for i := 1; i < len(tmpl.Config.Base); i++ {
 				isTemplate, _ = SeemsTemplateURL(tmpl.Config.Base[i].URL)
 				if !isTemplate {
@@ -84,7 +85,7 @@ func (tmpl *Template) embedAllBases(ctx context.Context, embedAll, defaultBase b
 				}
 			}
 			break
-			// TODO should we track embedding of template:// URLs so we can warn if we embed a non-template:// URL afterwards?
+			// TODO should we track embedding of template: URLs so we can warn if we embed a non-template: URL afterwards?
 		}
 
 		if seen[baseLocator.URL] {
@@ -106,7 +107,7 @@ func (tmpl *Template) embedAllBases(ctx context.Context, embedAll, defaultBase b
 	return nil
 }
 
-func (tmpl *Template) embedBase(ctx context.Context, baseLocator limayaml.LocatorWithDigest, embedAll bool, seen map[string]bool) error {
+func (tmpl *Template) embedBase(ctx context.Context, baseLocator limatype.LocatorWithDigest, embedAll bool, seen map[string]bool) error {
 	logrus.Debugf("Embedding base %q in template %q", baseLocator.URL, tmpl.Locator)
 	if err := tmpl.Unmarshal(); err != nil {
 		return err
@@ -179,9 +180,17 @@ func (tmpl *Template) mergeBase(base *Template) error {
 			tmpl.copyField(minimumLimaVersion, minimumLimaVersion)
 		}
 	}
-	if tmpl.Config.VMOpts.QEMU.MinimumVersion != nil && base.Config.VMOpts.QEMU.MinimumVersion != nil {
-		tmplVersion := *semver.New(*tmpl.Config.VMOpts.QEMU.MinimumVersion)
-		baseVersion := *semver.New(*base.Config.VMOpts.QEMU.MinimumVersion)
+	var tmplOpts limatype.QEMUOpts
+	if err := limayaml.Convert(tmpl.Config.VMOpts[limatype.QEMU], &tmplOpts, "vmOpts.qemu"); err != nil {
+		return err
+	}
+	var baseOpts limatype.QEMUOpts
+	if err := limayaml.Convert(base.Config.VMOpts[limatype.QEMU], &baseOpts, "vmOpts.qemu"); err != nil {
+		return err
+	}
+	if tmplOpts.MinimumVersion != nil && baseOpts.MinimumVersion != nil {
+		tmplVersion := *semver.New(*tmplOpts.MinimumVersion)
+		baseVersion := *semver.New(*baseOpts.MinimumVersion)
 		if tmplVersion.LessThan(baseVersion) {
 			const minimumQEMUVersion = "vmOpts.qemu.minimumVersion"
 			tmpl.copyField(minimumQEMUVersion, minimumQEMUVersion)
@@ -557,7 +566,7 @@ func encodeScriptReason(script string) string {
 	start := 0
 	line := 1
 	for i, r := range script {
-		if !(unicode.IsPrint(r) || r == '\n' || r == '\r' || r == '\t') {
+		if !(unicode.IsPrint(r) || r == '\n') {
 			return fmt.Sprintf("unprintable character %q at offset %d", r, i)
 		}
 		// maxLineLength includes final newline
@@ -625,7 +634,7 @@ func (tmpl *Template) embedAllScripts(ctx context.Context, embedAll bool) error
 			continue
 		}
 		// Don't overwrite existing script. This should throw an error during validation.
-		if p.Script != "" {
+		if p.Script == nil || *p.Script != "" {
 			continue
 		}
 		isTemplate, _ := SeemsTemplateURL(p.File.URL)
@@ -643,13 +652,18 @@ func (tmpl *Template) embedAllScripts(ctx context.Context, embedAll bool) error
 		}
 		newName := "script"
 		switch p.Mode {
-		case limayaml.ProvisionModeData:
+		case limatype.ProvisionModeData:
 			newName = "content"
 			if p.Content != nil {
 				continue
 			}
+		case limatype.ProvisionModeYQ:
+			newName = "expression"
+			if p.Expression != nil {
+				continue
+			}
 		default:
-			if p.Script != "" {
+			if p.Script != nil && *p.Script != "" {
 				continue
 			}
 		}
