@@ -10,8 +10,8 @@ import (
 
 	"gotest.tools/v3/assert"
 
-	"github.com/lima-vm/lima/pkg/limatmpl"
-	"github.com/lima-vm/lima/pkg/limayaml"
+	"github.com/lima-vm/lima/v2/pkg/limatmpl"
+	"github.com/lima-vm/lima/v2/pkg/limatype"
 )
 
 func TestInstNameFromImageURL(t *testing.T) {
@@ -48,7 +48,7 @@ func TestInstNameFromImageURL(t *testing.T) {
 		assert.Equal(t, name, "my-arch-aarch64")
 	})
 	t.Run("removes native arch", func(t *testing.T) {
-		arch := limayaml.NewArch(runtime.GOARCH)
+		arch := limatype.NewArch(runtime.GOARCH)
 		image := fmt.Sprintf("linux_cloudimg.base-%s.qcow2.gz", arch)
 		name := limatmpl.InstNameFromImageURL(image, arch)
 		assert.Equal(t, name, "linux")
@@ -64,7 +64,7 @@ func TestInstNameFromImageURL(t *testing.T) {
 }
 
 func TestSeemsTemplateURL(t *testing.T) {
-	arg := "template://foo/bar"
+	arg := "template:foo/bar"
 	t.Run(arg, func(t *testing.T) {
 		is, name := limatmpl.SeemsTemplateURL(arg)
 		assert.Equal(t, is, true)
@@ -96,7 +96,7 @@ func TestSeemsHTTPURL(t *testing.T) {
 	}
 	notHTTPURLs := []string{
 		"file:///foo",
-		"template://foo",
+		"template:foo",
 		"foo",
 	}
 	for _, arg := range notHTTPURLs {
@@ -114,7 +114,7 @@ func TestSeemsFileURL(t *testing.T) {
 	notFileURLs := []string{
 		"http://foo",
 		"https://foo",
-		"template://foo",
+		"template:foo",
 		"foo",
 	}
 	for _, arg := range notFileURLs {
