@@ -7,6 +7,7 @@ package osutil
 
 import (
 	"bytes"
+	"context"
 	"fmt"
 	"os"
 	"os/exec"
@@ -24,9 +25,9 @@ func SignalName(sig os.Signal) string {
 	return unix.SignalName(sig.(syscall.Signal))
 }
 
-func Sysctl(name string) (string, error) {
+func Sysctl(ctx context.Context, name string) (string, error) {
 	var stderrBuf bytes.Buffer
-	cmd := exec.Command("sysctl", "-n", name)
+	cmd := exec.CommandContext(ctx, "sysctl", "-n", name)
 	cmd.Stderr = &stderrBuf
 	stdout, err := cmd.Output()
 	if err != nil {
