@@ -19,5 +19,5 @@ func Control(_, _ string, c syscall.RawConn) (err error) {
 	if controlErr != nil {
 		err = controlErr
 	}
-	return
+	return err
 }
