@@ -5,6 +5,7 @@ package sshutil
 
 import (
 	"bytes"
+	"context"
 	"encoding/base64"
 	"encoding/binary"
 	"errors"
@@ -15,6 +16,7 @@ import (
 	"path/filepath"
 	"regexp"
 	"runtime"
+	"slices"
 	"strings"
 	"sync"
 	"time"
@@ -24,18 +26,25 @@ import (
 	"github.com/sirupsen/logrus"
 	"golang.org/x/sys/cpu"
 
-	"github.com/lima-vm/lima/pkg/ioutilx"
-	"github.com/lima-vm/lima/pkg/lockutil"
-	"github.com/lima-vm/lima/pkg/osutil"
-	"github.com/lima-vm/lima/pkg/store/dirnames"
-	"github.com/lima-vm/lima/pkg/store/filenames"
+	"github.com/lima-vm/lima/v2/pkg/ioutilx"
+	"github.com/lima-vm/lima/v2/pkg/limatype/dirnames"
+	"github.com/lima-vm/lima/v2/pkg/limatype/filenames"
+	"github.com/lima-vm/lima/v2/pkg/lockutil"
+	"github.com/lima-vm/lima/v2/pkg/osutil"
 )
 
 // Environment variable that allows configuring the command (alias) to execute
 // in place of the 'ssh' executable.
 const EnvShellSSH = "SSH"
 
-func SSHArguments() (arg0 string, arg0Args []string, err error) {
+type SSHExe struct {
+	Exe  string
+	Args []string
+}
+
+func NewSSHExe() (SSHExe, error) {
+	var sshExe SSHExe
+
 	if sshShell := os.Getenv(EnvShellSSH); sshShell != "" {
 		sshShellFields, err := shellwords.Parse(sshShell)
 		switch {
@@ -43,21 +52,21 @@ func SSHArguments() (arg0 string, arg0Args []string, err error) {
 			logrus.WithError(err).Warnf("Failed to split %s variable into shell tokens. "+
 				"Falling back to 'ssh' command", EnvShellSSH)
 		case len(sshShellFields) > 0:
-			arg0 = sshShellFields[0]
+			sshExe.Exe = sshShellFields[0]
 			if len(sshShellFields) > 1 {
-				arg0Args = sshShellFields[1:]
+				sshExe.Args = sshShellFields[1:]
 			}
+			return sshExe, nil
 		}
 	}
 
-	if arg0 == "" {
-		arg0, err = exec.LookPath("ssh")
-		if err != nil {
-			return "", []string{""}, err
-		}
+	executable, err := exec.LookPath("ssh")
+	if err != nil {
+		return SSHExe{}, err
 	}
+	sshExe.Exe = executable
 
-	return arg0, arg0Args, nil
+	return sshExe, nil
 }
 
 type PubKey struct {
@@ -83,7 +92,7 @@ func readPublicKey(f string) (PubKey, error) {
 //
 // When loadDotSSH is true, ~/.ssh/*.pub will be appended to make the VM accessible without specifying
 // an identity explicitly.
-func DefaultPubKeys(loadDotSSH bool) ([]PubKey, error) {
+func DefaultPubKeys(ctx context.Context, loadDotSSH bool) ([]PubKey, error) {
 	// Read $LIMA_HOME/_config/user.pub
 	configDir, err := dirnames.LimaConfigDir()
 	if err != nil {
@@ -101,12 +110,12 @@ func DefaultPubKeys(loadDotSSH bool) ([]PubKey, error) {
 			// no passphrase, no user@host comment
 			privPath := filepath.Join(configDir, filenames.UserPrivateKey)
 			if runtime.GOOS == "windows" {
-				privPath, err = ioutilx.WindowsSubsystemPath(privPath)
+				privPath, err = ioutilx.WindowsSubsystemPath(ctx, privPath)
 				if err != nil {
 					return err
 				}
 			}
-			keygenCmd := exec.Command("ssh-keygen", "-t", "ed25519", "-q", "-N", "",
+			keygenCmd := exec.CommandContext(ctx, "ssh-keygen", "-t", "ed25519", "-q", "-N", "",
 				"-C", "lima", "-f", privPath)
 			logrus.Debugf("executing %v", keygenCmd.Args)
 			if out, err := keygenCmd.CombinedOutput(); err != nil {
@@ -177,7 +186,7 @@ var sshInfo struct {
 //
 // The result always contains the IdentityFile option.
 // The result never contains the Port option.
-func CommonOpts(sshPath string, useDotSSH bool) ([]string, error) {
+func CommonOpts(ctx context.Context, sshExe SSHExe, useDotSSH bool) ([]string, error) {
 	configDir, err := dirnames.LimaConfigDir()
 	if err != nil {
 		return nil, err
@@ -188,7 +197,7 @@ func CommonOpts(sshPath string, useDotSSH bool) ([]string, error) {
 		return nil, err
 	}
 	var opts []string
-	idf, err := identityFileEntry(privateKeyPath)
+	idf, err := identityFileEntry(ctx, privateKeyPath)
 	if err != nil {
 		return nil, err
 	}
@@ -223,7 +232,7 @@ func CommonOpts(sshPath string, useDotSSH bool) ([]string, error) {
 				// Fail on permission-related and other path errors
 				return nil, err
 			}
-			idf, err = identityFileEntry(privateKeyPath)
+			idf, err = identityFileEntry(ctx, privateKeyPath)
 			if err != nil {
 				return nil, err
 			}
@@ -243,7 +252,7 @@ func CommonOpts(sshPath string, useDotSSH bool) ([]string, error) {
 
 	sshInfo.Do(func() {
 		sshInfo.aesAccelerated = detectAESAcceleration()
-		sshInfo.openSSH = detectOpenSSHInfo(sshPath)
+		sshInfo.openSSH = detectOpenSSHInfo(ctx, sshExe)
 	})
 
 	if sshInfo.openSSH.GSSAPISupported {
@@ -275,9 +284,9 @@ func CommonOpts(sshPath string, useDotSSH bool) ([]string, error) {
 	return opts, nil
 }
 
-func identityFileEntry(privateKeyPath string) (string, error) {
+func identityFileEntry(ctx context.Context, privateKeyPath string) (string, error) {
 	if runtime.GOOS == "windows" {
-		privateKeyPath, err := ioutilx.WindowsSubsystemPath(privateKeyPath)
+		privateKeyPath, err := ioutilx.WindowsSubsystemPath(ctx, privateKeyPath)
 		if err != nil {
 			return "", err
 		}
@@ -286,19 +295,56 @@ func identityFileEntry(privateKeyPath string) (string, error) {
 	return fmt.Sprintf(`IdentityFile="%s"`, privateKeyPath), nil
 }
 
+// DisableControlMasterOptsFromSSHArgs returns ssh args that disable ControlMaster, ControlPath, and ControlPersist.
+func DisableControlMasterOptsFromSSHArgs(sshArgs []string) []string {
+	argsForOverridingConfigFile := []string{
+		"-o", "ControlMaster=no",
+		"-o", "ControlPath=none",
+		"-o", "ControlPersist=no",
+	}
+	return slices.Concat(argsForOverridingConfigFile, removeOptsFromSSHArgs(sshArgs, "ControlMaster", "ControlPath", "ControlPersist"))
+}
+
+func removeOptsFromSSHArgs(sshArgs []string, removeOpts ...string) []string {
+	res := make([]string, 0, len(sshArgs))
+	isOpt := false
+	for _, arg := range sshArgs {
+		if isOpt {
+			isOpt = false
+			if !slices.ContainsFunc(removeOpts, func(opt string) bool {
+				return strings.HasPrefix(arg, opt)
+			}) {
+				res = append(res, "-o", arg)
+			}
+		} else if arg == "-o" {
+			isOpt = true
+		} else {
+			res = append(res, arg)
+		}
+	}
+	return res
+}
+
+// IsControlMasterExisting returns true if the control socket file exists.
+func IsControlMasterExisting(instDir string) bool {
+	controlSock := filepath.Join(instDir, filenames.SSHSock)
+	_, err := os.Stat(controlSock)
+	return err == nil
+}
+
 // SSHOpts adds the following options to CommonOptions: User, ControlMaster, ControlPath, ControlPersist.
-func SSHOpts(sshPath, instDir, username string, useDotSSH, forwardAgent, forwardX11, forwardX11Trusted bool) ([]string, error) {
+func SSHOpts(ctx context.Context, sshExe SSHExe, instDir, username string, useDotSSH, forwardAgent, forwardX11, forwardX11Trusted bool) ([]string, error) {
 	controlSock := filepath.Join(instDir, filenames.SSHSock)
 	if len(controlSock) >= osutil.UnixPathMax {
 		return nil, fmt.Errorf("socket path %q is too long: >= UNIX_PATH_MAX=%d", controlSock, osutil.UnixPathMax)
 	}
-	opts, err := CommonOpts(sshPath, useDotSSH)
+	opts, err := CommonOpts(ctx, sshExe, useDotSSH)
 	if err != nil {
 		return nil, err
 	}
 	controlPath := fmt.Sprintf(`ControlPath="%s"`, controlSock)
 	if runtime.GOOS == "windows" {
-		controlSock, err = ioutilx.WindowsSubsystemPath(controlSock)
+		controlSock, err = ioutilx.WindowsSubsystemPath(ctx, controlSock)
 		if err != nil {
 			return nil, err
 		}
@@ -332,6 +378,15 @@ func SSHArgsFromOpts(opts []string) []string {
 	return args
 }
 
+// SSHOptsRemovingControlPath removes ControlMaster, ControlPath, and ControlPersist options from SSH options.
+func SSHOptsRemovingControlPath(opts []string) []string {
+	// Create a copy of opts to avoid modifying the original slice, since slices.DeleteFunc modifies the slice in place.
+	copiedOpts := slices.Clone(opts)
+	return slices.DeleteFunc(copiedOpts, func(s string) bool {
+		return strings.HasPrefix(s, "ControlMaster") || strings.HasPrefix(s, "ControlPath") || strings.HasPrefix(s, "ControlPersist")
+	})
+}
+
 func ParseOpenSSHVersion(version []byte) *semver.Version {
 	regex := regexp.MustCompile(`(?m)^OpenSSH_(\d+\.\d+)(?:p(\d+))?\b`)
 	matches := regex.FindSubmatch(version)
@@ -361,18 +416,18 @@ var (
 	openSSHInfosRW sync.RWMutex
 )
 
-func detectOpenSSHInfo(ssh string) openSSHInfo {
+func detectOpenSSHInfo(ctx context.Context, sshExe SSHExe) openSSHInfo {
 	var (
 		info   openSSHInfo
 		exe    sshExecutable
 		stderr bytes.Buffer
 	)
-	path, err := exec.LookPath(ssh)
-	if err != nil {
-		logrus.Warnf("failed to find ssh executable: %v", err)
-	} else {
-		st, _ := os.Stat(path)
-		exe = sshExecutable{Path: path, Size: st.Size(), ModTime: st.ModTime()}
+	// Note: For SSH wrappers like "kitten ssh", os.Stat will check the wrapper
+	// executable (kitten) instead of the underlying ssh binary. This means
+	// cache invalidation won't work properly - ssh upgrades won't be detected
+	// since kitten's size/mtime won't change. This is probably acceptable.
+	if st, err := os.Stat(sshExe.Exe); err == nil {
+		exe = sshExecutable{Path: sshExe.Exe, Size: st.Size(), ModTime: st.ModTime()}
 		openSSHInfosRW.RLock()
 		info := openSSHInfos[exe]
 		openSSHInfosRW.RUnlock()
@@ -380,8 +435,10 @@ func detectOpenSSHInfo(ssh string) openSSHInfo {
 			return *info
 		}
 	}
+	sshArgs := append([]string{}, sshExe.Args...)
 	// -V should be last
-	cmd := exec.Command(path, "-o", "GSSAPIAuthentication=no", "-V")
+	sshArgs = append(sshArgs, "-o", "GSSAPIAuthentication=no", "-V")
+	cmd := exec.CommandContext(ctx, sshExe.Exe, sshArgs...)
 	cmd.Stderr = &stderr
 	if err := cmd.Run(); err != nil {
 		logrus.Warnf("failed to run %v: stderr=%q", cmd.Args, stderr.String())
@@ -398,8 +455,8 @@ func detectOpenSSHInfo(ssh string) openSSHInfo {
 	return info
 }
 
-func DetectOpenSSHVersion(ssh string) semver.Version {
-	return detectOpenSSHInfo(ssh).Version
+func DetectOpenSSHVersion(ctx context.Context, sshExe SSHExe) semver.Version {
+	return detectOpenSSHInfo(ctx, sshExe).Version
 }
 
 // detectValidPublicKey returns whether content represent a public key.
