@@ -5,6 +5,7 @@ package sysprof
 
 import (
 	"bytes"
+	"context"
 	"encoding/json"
 	"fmt"
 	"os/exec"
@@ -15,7 +16,7 @@ import (
 )
 
 var NetworkData = sync.OnceValues(func() ([]NetworkDataType, error) {
-	b, err := SystemProfiler("SPNetworkDataType")
+	b, err := SystemProfiler(context.Background(), "SPNetworkDataType")
 	if err != nil {
 		return nil, err
 	}
@@ -26,14 +27,14 @@ var NetworkData = sync.OnceValues(func() ([]NetworkDataType, error) {
 	return networkData.SPNetworkDataType, nil
 })
 
-func SystemProfiler(dataType string) ([]byte, error) {
+func SystemProfiler(ctx context.Context, dataType string) ([]byte, error) {
 	exe, err := exec.LookPath("system_profiler")
 	if err != nil {
 		// $PATH may lack /usr/sbin
 		exe = "/usr/sbin/system_profiler"
 	}
 	var stdout, stderr bytes.Buffer
-	cmd := exec.Command(exe, dataType, "-json")
+	cmd := exec.CommandContext(ctx, exe, dataType, "-json")
 	cmd.Stdout = &stdout
 	cmd.Stderr = &stderr
 	if err := cmd.Run(); err != nil {
